# -*- coding: utf-8 -*-
#_____________________________________________________________________________
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           Aaron Gibson   0x2C4B239DD876C9F6 <aagbsn@torproject.org>
#           Nick Mathewson 0x21194EBB165733EA <nickm@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, all entities within the AUTHORS file
#             (c) 2014-2015, Isis Lovecruft
# :license: see LICENSE for licensing information
#_____________________________________________________________________________

"""This module implements various methods for obtaining or creating CAPTCHAs.

**Module Overview:**

::

  captcha
   |- CaptchaExpired - Raised if a solution is given for a stale CAPTCHA.
   |- CaptchaKeyError - Raised if a CAPTCHA system's keys are invalid/missing.
   |- GimpCaptchaError - Raised when a Gimp CAPTCHA can't be retrieved.
   |
   \_ ICaptcha - Zope Interface specification for a generic CAPTCHA.
        |
      Captcha - Generic base class implementation for obtaining a CAPTCHA.
      |  |- image - The CAPTCHA image.
      |  |- challenge - A unique string associated with this CAPTCHA image.
      |  |- publicKey - The public key for this CAPTCHA system.
      |  |- secretKey - The secret key for this CAPTCHA system.
      |   \_ get() - Get a new pair of CAPTCHA image and challenge strings.
      |
      |- ReCaptcha - Obtain reCaptcha images and challenge strings.
      |   \_ get() - Request an image and challenge from a reCaptcha API server.
      |
      \_ GimpCaptcha - Class for obtaining a CAPTCHA from a local cache.
          |- hmacKey - A client-specific key for HMAC generation.
          |- cacheDir - The path to the local CAPTCHA cache directory.
          |- sched - A class for timing out CAPTCHAs after an interval.
          \_ get() - Get a CAPTCHA image from the cache and create a challenge.

..

There are two types of CAPTCHAs which BridgeDB knows how to serve: those
obtained by from a reCaptcha_ API server with
:class:`~bridgedb.captcha.Raptcha`, and those which have been generated with
gimp-captcha_ and then cached locally.

.. _reCaptcha : https://code.google.com/p/recaptcha/
.. _gimp-captcha: https://github.com/isislovecruft/gimp-captcha
"""

from base64 import urlsafe_b64encode
from base64 import urlsafe_b64decode

import logging
import random
import os
import time
import urllib2

from BeautifulSoup import BeautifulSoup

from zope.interface import Interface, Attribute, implements

from bridgedb import crypto
from bridgedb import schedule
from bridgedb.txrecaptcha import API_SSL_SERVER


class CaptchaExpired(ValueError):
    """Raised when a client's CAPTCHA is too stale."""

class CaptchaKeyError(Exception):
    """Raised if a CAPTCHA system's keys are invalid or missing."""

class GimpCaptchaError(Exception):
    """General exception raised when a Gimp CAPTCHA cannot be retrieved."""


class ICaptcha(Interface):
    """Interface specification for CAPTCHAs."""

    image = Attribute(
        "A string containing the contents of a CAPTCHA image file.")
    challenge = Attribute(
        "A unique string associated with the dispursal of this CAPTCHA.")
    publicKey = Attribute(
        "A public key used for encrypting CAPTCHA challenge strings.")
    secretKey = Attribute(
        "A private key used for decrypting challenge strings during CAPTCHA"
        "solution verification.")

    def get():
        """Retrieve a new CAPTCHA image."""


class Captcha(object):
    """A generic CAPTCHA base class.

    :ivar str image: The CAPTCHA image.
    :ivar str challenge: A challenge string which should permit checking of
        the client's CAPTCHA solution in some manner. In stateless protocols
        such as HTTP, this should be passed along to the client with the
        CAPTCHA image.
    :ivar publicKey: A public key used for encrypting CAPTCHA challenge strings.
    :ivar secretKey: A private key used for decrypting challenge strings during
        CAPTCHA solution verification.
    """
    implements(ICaptcha)

    def __init__(self, publicKey=None, secretKey=None):
        """Obtain a new CAPTCHA for a client."""
        self.image = None
        self.challenge = None
        self.publicKey = publicKey
        self.secretKey = secretKey

    def get(self):
        """Retrieve a new CAPTCHA image and its associated challenge string.

        The image and challenge will be stored as :ivar:`image` and
        :ivar:`challenge, respectively.
        """
        self.image = None
        self.challenge = None


class ReCaptcha(Captcha):
    """A CAPTCHA obtained from a remote reCaptcha_ API server.

    :ivar str image: The CAPTCHA image.
    :ivar str challenge: The ``'recaptcha_challenge_response'`` HTTP form
        field to pass to the client, along with the CAPTCHA image. See
        :doc:`BridgeDB's captcha.html <templates/captcha.html>` Mako_ template
        for an example usage.
    :ivar str publicKey: The public reCaptcha API key.
    :ivar str secretKey: The private reCaptcha API key.

    .. _reCaptcha: https://code.google.com/p/recaptcha/
    .. _Mako: http://docs.makotemplates.org/en/latest/syntax.html#page
    """

    def __init__(self, publicKey=None, secretKey=None):
        """Create a new ReCaptcha CAPTCHA.

        :param str publicKey: The public reCaptcha API key.
        :param str secretKey: The private reCaptcha API key.
        """
        super(ReCaptcha, self).__init__(publicKey=publicKey,
                                        secretKey=secretKey)

    def get(self):
        """Retrieve a CAPTCHA from the reCaptcha API server.

        This simply requests a new CAPTCHA from
        ``recaptcha.client.captcha.API_SSL_SERVER`` and parses the returned
        HTML to extract the CAPTCHA image and challenge string. The image is
        stored at ``ReCaptcha.image`` and the challenge string at
        ``ReCaptcha.challenge``.

        :raises CaptchaKeyError: If either the :ivar:`publicKey` or
            :ivar:`secretKey` are missing.
        :raises HTTPError: If the server returned any HTTP error status code.
        """
        if not self.publicKey or not self.secretKey:
            raise CaptchaKeyError('You must supply recaptcha API keys')

        urlbase = API_SSL_SERVER
        form = "/noscript?k=%s" % self.publicKey

        # Extract and store image from recaptcha
        html = urllib2.urlopen(urlbase + form).read()
        # FIXME: The remaining lines currently cannot be reliably unit tested:
        soup = BeautifulSoup(html)                           # pragma: no cover
        imgurl = urlbase + "/" +  soup.find('img')['src']    # pragma: no cover
        cField = soup.find(                                  # pragma: no cover
            'input', {'name': 'recaptcha_challenge_field'})  # pragma: no cover
        self.challenge = str(cField['value'])                # pragma: no cover
        self.image = urllib2.urlopen(imgurl).read()          # pragma: no cover


class GimpCaptcha(Captcha):
    """A locally cached CAPTCHA image which was created with gimp-captcha_.

    :ivar str secretKey: A PKCS#1 OAEP-padded, private RSA key, used for
        verifying the client's solution to the CAPTCHA.
    :ivar str publickey: A PKCS#1 OAEP-padded, public RSA key. This is used to
        hide the correct CAPTCHA solution within the
        ``captcha_challenge_field`` HTML form field. That form field is given
        to the a client along with the :ivar:`image` during the initial
        CAPTCHA request, and the client *should* give it back to us later
        during the CAPTCHA solution verification step.
    :ivar bytes hmacKey: A client-specific HMAC secret key.
    :ivar str cacheDir: The local directory which pre-generated CAPTCHA images
        have been stored in. This can be set via the ``GIMP_CAPTCHA_DIR``
        setting in the config file.
    :type sched: :class:`bridgedb.schedule.ScheduledInterval`
    :ivar sched: An time interval. After this much time has passed, the
        CAPTCHA is considered stale, and all solutions are considered invalid
        regardless of their correctness.

    .. _gimp-captcha: https://github.com/isislovecruft/gimp-captcha
    """

    sched = schedule.ScheduledInterval('minutes', 30)

    def __init__(self, publicKey=None, secretKey=None, hmacKey=None,
                 cacheDir=None):
        """Create a ``GimpCaptcha`` which retrieves images from **cacheDir**.

        :param str publickey: A PKCS#1 OAEP-padded, public RSA key, used for
            creating the ``captcha_challenge_field`` string to give to a
            client.
        :param str secretKey: A PKCS#1 OAEP-padded, private RSA key, used for
            verifying the client's solution to the CAPTCHA.
        :param bytes hmacKey: A client-specific HMAC secret key.
        :param str cacheDir: The local directory which pre-generated CAPTCHA
            images have been stored in. This can be set via the
            ``GIMP_CAPTCHA_DIR`` setting in the config file.
        :raises GimpCaptchaError: if :ivar:`cacheDir` is not a directory.
        :raises CaptchaKeyError: if any of :ivar:`secretKey`,
            :ivar:`publicKey`, or :ivar:`hmacKey` are invalid or missing.
        """
        if not cacheDir or not os.path.isdir(cacheDir):
            raise GimpCaptchaError("Gimp captcha cache isn't a directory: %r"
                                   % cacheDir)
        if not (publicKey and secretKey and hmacKey):
            raise CaptchaKeyError(
                "Invalid key supplied to GimpCaptcha: SK=%r PK=%r HMAC=%r"
                % (secretKey, publicKey, hmacKey))

        super(GimpCaptcha, self).__init__(publicKey=publicKey,
                                          secretKey=secretKey)
        self.hmacKey = hmacKey
        self.cacheDir = cacheDir
        self.answer = None

    @classmethod
    def check(cls, challenge, solution, secretKey, hmacKey):
        """Check a client's CAPTCHA **solution** against the **challenge**.

        :param str challenge: The contents of the
            ``'captcha_challenge_field'`` HTTP form field.
        :param str solution: The client's proposed solution to the CAPTCHA
            that they were presented with.
        :param str secretKey: A PKCS#1 OAEP-padded, private RSA key, used for
            verifying the client's solution to the CAPTCHA.
        :param bytes hmacKey: A private key for generating HMACs.
        :raises CaptchaExpired: if the **solution** was for a stale CAPTCHA.
        :rtype: bool
        :returns: ``True`` if the CAPTCHA solution was correct and not
            stale. ``False`` otherwise.
        """
        hmacIsValid = False

        if not solution:
            return hmacIsValid

        logging.debug("Checking CAPTCHA solution %r against challenge %r"
                      % (solution, challenge))
        try:
            decoded = urlsafe_b64decode(challenge)
            hmacFromBlob = decoded[:20]
            encBlob = decoded[20:]
            hmacNew = crypto.getHMAC(hmacKey, encBlob)
            hmacIsValid = hmacNew == hmacFromBlob
        except Exception:
            return False
        finally:
            if hmacIsValid:
                try:
                    answerBlob = secretKey.decrypt(encBlob)

                    timestamp = answerBlob[:12].lstrip('0')
                    then = cls.sched.nextIntervalStarts(int(timestamp))
                    now = int(time.time())
                    answer = answerBlob[12:]
                except Exception as error:
                    logging.warn(error.message)
                else:
                    # If the beginning of the 'next' interval (the interval
                    # after the one when the CAPTCHA timestamp was created)
                    # has already passed, then the CAPTCHA is stale.
                    if now >= then:
                        exp = schedule.fromUnixSeconds(then).isoformat(sep=' ')
                        raise CaptchaExpired("Solution %r was for a CAPTCHA "
                                             "which already expired at %s."
                                             % (solution, exp))
                    if solution.lower() == answer.lower():
                        return True
            return False

    def createChallenge(self, answer):
        """Encrypt-then-HMAC a timestamp plus the CAPTCHA **answer**.

        A challenge string consists of a URL-safe, base64-encoded string which
        contains an ``HMAC`` concatenated with an ``ENC_BLOB``, in the
        following form::

            CHALLENGE := B64( HMAC | ENC_BLOB )
            ENC_BLOB := RSA_ENC( ANSWER_BLOB )
            ANSWER_BLOB := ( TIMESTAMP | ANSWER )

        where
          * ``B64`` is a URL-safe base64-encode function,
          * ``RSA_ENC`` is the PKCS#1 RSA-OAEP encryption function,
          * and the remaining feilds are specified as follows:

        +-------------+--------------------------------------------+----------+
        | Field       | Description                                | Length   |
        +=============+============================================+==========+
        | HMAC        | An HMAC of the ``ENC_BLOB``, created with  | 20 bytes |
        |             | the client-specific :ivar:`hmacKey`, by    |          |
        |             | applying :func:`~crypto.getHMAC` to the    |          |
        |             | ``ENC_BLOB``.                              |          |
        +-------------+--------------------------------------------+----------+
        | ENC_BLOB    | An encrypted ``ANSWER_BLOB``, created with | varies   |
        |             | a PKCS#1 OAEP-padded RSA :ivar:`publicKey`.|          |
        +-------------+--------------------------------------------+----------+
        | ANSWER_BLOB | Contains the concatenated ``TIMESTAMP``    | varies   |
        |             | and ``ANSWER``.                            |          |
        +-------------+--------------------------------------------+----------+
        | TIMESTAMP   | A Unix Epoch timestamp, in seconds,        | 12 bytes |
        |             | left-padded with "0"s.                     |          |
        +-------------+--------------------------------------------+----------+
        | ANSWER      | A string containing answer to this         | 8 bytes  |
        |             | CAPTCHA :ivar:`image`.                     |          |
        +-------------+--------------------------------------------+----------+

        The steps taken to produce a ``CHALLENGE`` are then:

          1. Create a ``TIMESTAMP``, and pad it on the left with ``0``s to 12
             bytes in length.

          2. Next, take the **answer** to this CAPTCHA :ivar:`image: and
             concatenate the padded ``TIMESTAMP`` and the ``ANSWER``, forming
             an ``ANSWER_BLOB``.

          3. Encrypt the resulting ``ANSWER_BLOB`` to :ivar:`publicKey` to
             create the ``ENC_BLOB``.

          4. Use the client-specific :ivar:`hmacKey` to apply the
             :func:`~crypto.getHMAC` function to the ``ENC_BLOB``, obtaining
             an ``HMAC``.

          5. Create the final ``CHALLENGE`` string by concatenating the
             ``HMAC`` and ``ENC_BLOB``, then base64-encoding the result.

        :param str answer: The answer to a CAPTCHA.
        :rtype: str
        :returns: A challenge string.
        """
        timestamp = str(int(time.time())).zfill(12)
        blob = timestamp + answer
        encBlob = self.publicKey.encrypt(blob)
        hmac = crypto.getHMAC(self.hmacKey, encBlob)
        challenge = urlsafe_b64encode(hmac + encBlob)
        return challenge

    def get(self):
        """Get a random CAPTCHA from the cache directory.

        This chooses a random CAPTCHA image file from the cache directory, and
        reads the contents of the image into a string. Next, it creates a
        challenge string for the CAPTCHA, via :meth:`createChallenge`.

        :raises GimpCaptchaError: if the chosen CAPTCHA image file could not
            be read, or if the **cacheDir** is empty.
        :rtype: tuple
        :returns: A 2-tuple containing the image file contents as a string,
            and a challenge string (used for checking the client's solution).
        """
        try:
            imageFilename = random.choice(os.listdir(self.cacheDir))
            imagePath = os.path.join(self.cacheDir, imageFilename)
            with open(imagePath) as imageFile:
                self.image = imageFile.read()
        except IndexError:
            raise GimpCaptchaError("CAPTCHA cache dir appears empty: %r"
                                   % self.cacheDir)
        except (OSError, IOError):
            raise GimpCaptchaError("Could not read Gimp captcha image file: %r"
                                   % imageFilename)

        self.answer = imageFilename.rsplit(os.path.extsep, 1)[0]
        self.challenge = self.createChallenge(self.answer)

        return (self.image, self.challenge)



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_Dist -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Nick Mathewson
#           Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           Matthew Finkel 0x017DD169EA793BE2 <sysrqb@torproject.org>
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2013-2015, Matthew Finkel
#             (c) 2007-2015, The Tor Project, Inc.
# :license: 3-Clause BSD, see LICENSE for licensing information

"""This module has functions to decide which bridges to hand out to whom."""

import logging
import re
import time

from ipaddr import IPv6Address
from ipaddr import IPAddress

import bridgedb.Bridges
import bridgedb.Storage

from bridgedb.crypto import getHMAC
from bridgedb.crypto import getHMACFunc
from bridgedb.Filters import filterAssignBridgesToRing
from bridgedb.Filters import filterBridgesByRules
from bridgedb.Filters import filterBridgesByIP4
from bridgedb.Filters import filterBridgesByIP6
from bridgedb.parse import addr
from bridgedb.parse.addr import UnsupportedDomain
from bridgedb.safelog import logSafely


MAX_EMAIL_RATE = 3*3600

class IgnoreEmail(addr.BadEmail):
    """Raised when we get requests from this address after rate warning."""

class TooSoonEmail(addr.BadEmail):
    """Raised when we got a request from this address too recently."""

class EmailRequestedHelp(Exception):
    """Raised when a client has emailed requesting help."""

class EmailRequestedKey(Exception):
    """Raised when an incoming email requested a copy of our GnuPG keys."""


def uniformMap(ip):
    """Map an IP to an arbitrary 'area' string, such that any two /24 addresses
    get the same string.

    >>> from bridgedb import Dist
    >>> Dist.uniformMap('1.2.3.4')
    '1.2.3'

    :param str ip: A string representing an IPv4 or IPv6 address.
    """
    if type(IPAddress(ip)) is IPv6Address:
        return ":".join(IPv6Address(ip).exploded.split(':')[:4])
    else:
        return ".".join(ip.split(".")[:3])

def getNumBridgesPerAnswer(ring, max_bridges_per_answer=3):
    if len(ring) < 20:
        n_bridges_per_answer = 1
    if 20 <= len(ring) < 100:
        n_bridges_per_answer = min(2, max_bridges_per_answer)
    if len(ring) >= 100:
        n_bridges_per_answer = max_bridges_per_answer

    logging.debug("Returning %d bridges from ring of len: %d" %
                  (n_bridges_per_answer, len(ring)))

    return n_bridges_per_answer

class Distributor(bridgedb.Bridges.BridgeHolder):
    """Distributes bridges to clients."""

    def __init__(self):
        super(Distributor, self).__init__()

    def setDistributorName(self, name):
        """Set a **name** for identifying this distributor.

        This is used to identify the distributor in the logs; the **name**
        doesn't necessarily need to be unique. The hashrings created for this
        distributor will be named after this distributor's name in
        :meth:`propopulateRings`, and any sub hashrings of each of those
        hashrings will also carry that name.

        >>> from bridgedb import Dist
        >>> ipDist = Dist.IPBasedDistributor(Dist.uniformMap,
        ...                                  5,
        ...                                  'fake-hmac-key')
        >>> ipDist.setDistributorName('HTTPS Distributor')
        >>> ipDist.prepopulateRings()
        >>> hashrings = ipDist.splitter.filterRings
        >>> firstSubring = hashrings.items()[0][1][1]
        >>> assert firstSubring.name

        :param str name: A name for this distributor.
        """
        self.name = name
        self.splitter.distributorName = name


class IPBasedDistributor(Distributor):
    """A Distributor that hands out bridges based on the IP address of an
    incoming request and the current time period.

    :ivar areaOrderHmac: An HMAC function used to order areas within rings.
    :ivar areaClusterHmac: An HMAC function used to assign areas to rings.
    :ivar list rings: A list of :class:`bridgedb.Bridges.BridgeHolder`
        hashrings, one for each area in the ``areaMapper``. Every inserted
        bridge will go into one of these rings, and every area is associated
        with one.
    :ivar categories: DOCDOC See :param:`ipCategories`.
    :type splitter: :class:`bridgedb.Bridges.FixedBridgeSplitter`
    :ivar splitter: A hashring that assigns bridges to subrings with fixed
        proportions. Used to assign bridges into the subrings of this
        distributor.
    """

    def __init__(self, areaMapper, nClusters, key,
                 ipCategories=None, answerParameters=None):
        """Create a Distributor that decides which bridges to distribute based
        upon the client's IP address and the current time.

        :type areaMapper: callable
        :param areaMapper: A function that maps IP addresses arbitrarily to
            strings, such that addresses which map to identical strings are
            considered to be in the same "area" (for some arbitrary definition
            of "area"). See :func:`bridgedb.Dist.uniformMap` for an example.
        :param integer nClusters: The number of clusters to group IP addresses
            into. Note that if PROXY_LIST_FILES is set in bridgedb.conf, then
            the actual number of clusters is one higher than ``nClusters``,
            because the set of known open proxies constitutes its own
            category.
            DOCDOC What exactly does a cluster *do*?
        :param bytearray key: The master HMAC key for this distributor. All
            added bridges are HMACed with this key in order to place them into
            the hashrings.
        :type ipCategories: iterable or None
        :param ipCategories: DOCDOC
        :type answerParameters: :class:`bridgedb.Bridges.BridgeRingParameters`
        :param answerParameters: A mechanism for ensuring that the set of
            bridges that this distributor answers a client with fit certain
            parameters, i.e. that an answer has "at least two obfsproxy
            bridges" or "at least one bridge on port 443", etc.
        """
        self.areaMapper = areaMapper
        self.nClusters = nClusters
        self.answerParameters = answerParameters

        if not ipCategories:
            ipCategories = []
        if not answerParameters:
            answerParameters = []
        self.rings = []

        self.categories = []
        for c in ipCategories:
            self.categories.append(c)

        key2 = getHMAC(key, "Assign-Bridges-To-Rings")
        key3 = getHMAC(key, "Order-Areas-In-Rings")
        self.areaOrderHmac = getHMACFunc(key3, hex=False)
        key4 = getHMAC(key, "Assign-Areas-To-Rings")
        self.areaClusterHmac = getHMACFunc(key4, hex=True)

        # add splitter and cache the default rings
        # plus leave room for dynamic filters
        #
        # XXX Why is the "extra room" hardcoded to be 5? Shouldn't it be some
        #     fraction of the number of clusters/categories? --isis
        ring_cache_size  = self.nClusters + len(ipCategories) + 5
        self.splitter = bridgedb.Bridges.FilteredBridgeSplitter(
            key2, max_cached_rings=ring_cache_size)
        logging.debug("Added splitter %s to IPBasedDistributor."
                      % self.splitter.__class__)

        self.setDistributorName('HTTPS')

    def prepopulateRings(self):
        logging.info("Prepopulating %s distributor hashrings..." % self.name)
        # populate all rings (for dumping assignments and testing)
        for filterFn in [None, filterBridgesByIP4, filterBridgesByIP6]:
            n = self.nClusters
            for category in self.categories:
                g = filterAssignBridgesToRing(self.splitter.hmac,
                                              self.nClusters +
                                              len(self.categories),
                                              n)
                bridgeFilterRules = [g]
                if filterFn:
                    bridgeFilterRules.append(filterFn)
                ruleset = frozenset(bridgeFilterRules)
                key1 = getHMAC(self.splitter.key,
                               "Order-Bridges-In-Ring-%d" % n)
                n += 1
                ring = bridgedb.Bridges.BridgeRing(key1, self.answerParameters)
                ring.setName('{0} Ring'.format(self.name))
                self.splitter.addRing(ring,
                                      ruleset,
                                      filterBridgesByRules(bridgeFilterRules),
                                      populate_from=self.splitter.bridges)


            # populate all ip clusters
            for clusterNum in xrange(self.nClusters):
                g = filterAssignBridgesToRing(self.splitter.hmac,
                                              self.nClusters +
                                              len(self.categories),
                                              clusterNum)
                bridgeFilterRules = [g]
                if filterFn:
                    bridgeFilterRules.append(filterFn)
                ruleset = frozenset(bridgeFilterRules)
                key1 = getHMAC(self.splitter.key,
                               "Order-Bridges-In-Ring-%d" % clusterNum)
                ring = bridgedb.Bridges.BridgeRing(key1, self.answerParameters)
                self.splitter.addRing(ring,
                                      ruleset,
                                      filterBridgesByRules(bridgeFilterRules),
                                      populate_from=self.splitter.bridges)

    def clear(self):
        self.splitter.clear()

    def insert(self, bridge):
        """Assign a bridge to this distributor."""
        self.splitter.insert(bridge)

    def getBridgesForIP(self, ip, epoch, N=1, countryCode=None,
                        bridgeFilterRules=None):
        """Return a list of bridges to give to a user.

        :param str ip: The user's IP address, as a dotted quad.
        :param str epoch: The time period when we got this request.  This can
                          be any string, so long as it changes with every
                          period.
        :param int N: The number of bridges to try to give back. (default: 1)
        :param str countryCode: DOCDOC (default: None)
        :param list bridgeFilterRules: A list of callables used filter the
                                       bridges returned in the response to the
                                       client. See :mod:`~bridgedb.Filters`.
        :rtype: list
        :return: A list of :class:`~bridgedb.Bridges.Bridge`s to include in
                 the response. See
                 :meth:`bridgedb.HTTPServer.WebResource.getBridgeRequestAnswer`
                 for an example of how this is used.
        """
        logging.info("Attempting to return %d bridges to client %s..."
                     % (N, ip))

        if not bridgeFilterRules:
            bridgeFilterRules=[]

        if not len(self.splitter):
            logging.warn("Bailing! Splitter has zero bridges!")
            return []

        logging.debug("Bridges in splitter:\t%d" % len(self.splitter))
        logging.debug("Client request epoch:\t%s" % epoch)
        logging.debug("Active bridge filters:\t%s"
                      % ' '.join([x.func_name for x in bridgeFilterRules]))

        area = self.areaMapper(ip)
        logging.debug("IP mapped to area:\t%s"
                      % logSafely("{0}.0/24".format(area)))

        key1 = ''
        pos = 0
        n = self.nClusters

        # only one of ip categories or area clustering is active
        # try to match the request to an ip category
        for category in self.categories:
            # IP Categories
            if category.contains(ip):
                g = filterAssignBridgesToRing(self.splitter.hmac,
                                              self.nClusters +
                                              len(self.categories),
                                              n)
                bridgeFilterRules.append(g)
                logging.info("category<%s>%s", epoch, logSafely(area))
                pos = self.areaOrderHmac("category<%s>%s" % (epoch, area))
                key1 = getHMAC(self.splitter.key,
                               "Order-Bridges-In-Ring-%d" % n)
                break
            n += 1

        # if no category matches, use area clustering
        else:
            # IP clustering
            h = int( self.areaClusterHmac(area)[:8], 16)
            # length of numClusters
            clusterNum = h % self.nClusters

            g = filterAssignBridgesToRing(self.splitter.hmac,
                                          self.nClusters +
                                          len(self.categories),
                                          clusterNum)
            bridgeFilterRules.append(g)
            pos = self.areaOrderHmac("<%s>%s" % (epoch, area))
            key1 = getHMAC(self.splitter.key,
                           "Order-Bridges-In-Ring-%d" % clusterNum)

        # try to find a cached copy
        ruleset = frozenset(bridgeFilterRules)

        # See if we have a cached copy of the ring,
        # otherwise, add a new ring and populate it
        if ruleset in self.splitter.filterRings.keys():
            logging.debug("Cache hit %s" % ruleset)
            _,ring = self.splitter.filterRings[ruleset]

        # else create the ring and populate it
        else:
            logging.debug("Cache miss %s" % ruleset)
            ring = bridgedb.Bridges.BridgeRing(key1, self.answerParameters)
            self.splitter.addRing(ring,
                                  ruleset,
                                  filterBridgesByRules(bridgeFilterRules),
                                  populate_from=self.splitter.bridges)

        # get an appropriate number of bridges
        numBridgesToReturn = getNumBridgesPerAnswer(ring,
                                                    max_bridges_per_answer=N)
        answer = ring.getBridges(pos, numBridgesToReturn)
        return answer

    def __len__(self):
        return len(self.splitter)

    def dumpAssignments(self, f, description=""):
        self.splitter.dumpAssignments(f, description)

class EmailBasedDistributor(Distributor):
    """Object that hands out bridges based on the email address of an incoming
    request and the current time period.

    :type splitter: :class:`~bridgedb.Bridges.BridgeRing`
    :ivar splitter: A hashring to hold all the bridges we hand out.
    """

    def __init__(self, key, domainmap, domainrules,
                 answerParameters=None, whitelist=None):
        """Create a bridge distributor which uses email.

        :type emailHmac: callable
        :param emailHmac: An hmac function used to order email addresses
            within a ring. See :func:`~bridgedb.crypto.getHMACFunc`.
        :param dict domainmap: A map from lowercase domains that we support
            mail from to their canonical forms. See `EMAIL_DOMAIN_MAP` option
            in `bridgedb.conf`.
        :param domainrules: DOCDOC
        :param answerParameters: DOCDOC
        :type whitelist: dict or ``None``
        :param whitelist: A dictionary that maps whitelisted email addresses
            to GnuPG fingerprints.
        """
        key1 = getHMAC(key, "Map-Addresses-To-Ring")
        self.emailHmac = getHMACFunc(key1, hex=False)

        key2 = getHMAC(key, "Order-Bridges-In-Ring")
        # XXXX clear the store when the period rolls over!
        self.domainmap = domainmap
        self.domainrules = domainrules
        self.whitelist = whitelist or dict()
        self.answerParameters = answerParameters

        #XXX cache options not implemented
        self.splitter = bridgedb.Bridges.FilteredBridgeSplitter(
            key2, max_cached_rings=5)

        self.setDistributorName('Email')

    def clear(self):
        self.splitter.clear()
        #self.ring.clear() # should be take care of by above

    def insert(self, bridge):
        """Assign a bridge to this distributor."""
        self.splitter.insert(bridge)

    def getBridgesForEmail(self, emailaddress, epoch, N=1, parameters=None,
                           countryCode=None, bridgeFilterRules=None):
        """Return a list of bridges to give to a user.

        :param str emailaddress: The user's email address, as given in a
            :header:`From:` line.
        :param epoch: The time period when we got this request. This can be
            any string, so long as it changes with every period.
        :param int N: The number of bridges to try to give back.
        :param parameters: DOCDOC
        :param countryCode: DOCDOC
        :param bridgeFilterRules: DOCDOC
        """
        if not bridgeFilterRules:
            bridgeFilterRules=[]
        now = time.time()

        # All checks on the email address, such as checks for whitelisting and
        # canonicalization of domain name, are done in
        # :meth:`bridgedb.email.autoresponder.getMailTo` and
        # :meth:`bridgedb.email.autoresponder.SMTPAutoresponder.runChecks`.
        if not emailaddress:
            logging.error(("%s distributor can't get bridges for blank email "
                           "address!") % (self.name, emailaddress))
            return []

        with bridgedb.Storage.getDB() as db:
            wasWarned = db.getWarnedEmail(emailaddress)
            lastSaw = db.getEmailTime(emailaddress)

            logging.info("Attempting to return for %d bridges for %s..."
                         % (N, emailaddress))

            if lastSaw is not None:
                if emailaddress in self.whitelist.keys():
                    logging.info(("Whitelisted email address %s was last seen "
                                  "%d seconds ago.")
                                 % (emailaddress, now - lastSaw))
                elif (lastSaw + MAX_EMAIL_RATE) >= now:
                    wait = (lastSaw + MAX_EMAIL_RATE) - now
                    logging.info("Client %s must wait another %d seconds."
                                 % (emailaddress, wait))
                    if wasWarned:
                        raise IgnoreEmail("Client was warned.", emailaddress)
                    else:
                        logging.info("Sending duplicate request warning.")
                        db.setWarnedEmail(emailaddress, True, now)
                        db.commit()
                        raise TooSoonEmail("Must wait %d seconds" % wait,
                                           emailaddress)

            # warning period is over
            elif wasWarned:
                db.setWarnedEmail(emailaddress, False)

            pos = self.emailHmac("<%s>%s" % (epoch, emailaddress))

            ring = None
            ruleset = frozenset(bridgeFilterRules)
            if ruleset in self.splitter.filterRings.keys():
                logging.debug("Cache hit %s" % ruleset)
                _, ring = self.splitter.filterRings[ruleset]
            else:
                # cache miss, add new ring
                logging.debug("Cache miss %s" % ruleset)

                # add new ring
                key1 = getHMAC(self.splitter.key,
                                                 "Order-Bridges-In-Ring")
                ring = bridgedb.Bridges.BridgeRing(key1, self.answerParameters)
                # debug log: cache miss
                self.splitter.addRing(ring, ruleset,
                                      filterBridgesByRules(bridgeFilterRules),
                                      populate_from=self.splitter.bridges)

            numBridgesToReturn = getNumBridgesPerAnswer(ring,
                                                        max_bridges_per_answer=N)
            result = ring.getBridges(pos, numBridgesToReturn)

            db.setEmailTime(emailaddress, now)
            db.commit()

        return result

    def __len__(self):
        return len(self.splitter)

    def cleanDatabase(self):
        with bridgedb.Storage.getDB() as db:
            try:
                db.cleanEmailedBridges(time.time() - MAX_EMAIL_RATE)
                db.cleanWarnedEmails(time.time() - MAX_EMAIL_RATE)
            except:
                db.rollback()
                raise
            else:
                db.commit()

    def dumpAssignments(self, f, description=""):
        self.splitter.dumpAssignments(f, description)

    def prepopulateRings(self):
        # populate all rings (for dumping assignments and testing)
        for filterFn in [filterBridgesByIP4, filterBridgesByIP6]:
            ruleset = frozenset([filterFn])
            key1 = getHMAC(self.splitter.key, "Order-Bridges-In-Ring")
            ring = bridgedb.Bridges.BridgeRing(key1, self.answerParameters)
            self.splitter.addRing(ring, ruleset,
                                  filterBridgesByRules([filterFn]),
                                  populate_from=self.splitter.bridges)



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_translations -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: 3-Clause BSD, see LICENSE for licensing information

import gettext
import logging
import os
import re

from bridgedb import _langs
from bridgedb import safelog
from bridgedb.parse import headers


TRANSLATIONS_DIR = os.path.join(os.path.dirname(__file__), 'i18n')


def getFirstSupportedLang(langs):
    """Return the first language in **langs** that we support.

    :param list langs: All requested languages
    :rtype: str
    :returns: A country code for the client's preferred language.
    """
    lang = 'en-US'
    supported = _langs.get_langs()

    for l in langs:
        if l in supported:
            lang = l
            break
    return lang

def getLocaleFromHTTPRequest(request):
    """Retrieve the languages from an HTTP ``Accept-Language:`` header.

    Parse the languages from the header, use them to install a
    ``gettext.translation`` chain via :func:`installTranslations`, and lastly
    return the requested languages.

    :type request: :api:`twisted.web.server.Request`
    :param request: An incoming request from a client.
    :rtype: list
    :returns: All requested languages.
    """
    header = request.getHeader('accept-language')
    if header is None:
        logging.debug("Client sent no 'Accept-Language' header. Using fallback.")
        header = 'en,en-US'

    langs = headers.parseAcceptLanguage(header)
    if not safelog.safe_logging:  # pragma: no cover
        logging.debug("Client Accept-Language (top 5): %s" % langs[:5])

    # Check if we got a ?lang=foo argument, and if we did, insert it first
    chosenLang = request.args.get("lang", [None,])[0]
    if chosenLang:
        logging.debug("Client requested language: %r" % chosenLang)
        langs.insert(0, chosenLang)

    installTranslations(langs)
    return langs

def getLocaleFromPlusAddr(address):
    """See whether the user sent his email to a 'plus' address, for instance to
    bridges+fa@bridges.torproject.org. Plus addresses are the current
    mechanism to set the reply language.
    """
    replyLocale = "en"
    r = '.*(<)?(\w+\+(\w+)@\w+(?:\.\w+)+)(?(1)>)'
    match = re.match(r, address)
    if match:
        replyLocale = match.group(3)

    return replyLocale

def installTranslations(langs):
    """Create a ``gettext.translation`` chain for all **langs**.

    Attempt to install the first language in the **langs** list. If that
    fails, we receive a ``gettext.NullTranslation`` object, and if it worked
    then we have a ``gettext.GNUTranslation`` object. Whichever one we end up
    with, get the other languages and add them as fallbacks to the
    first. Lastly, install this chain of translations.

    :param list langs: A list of language codes.
    :returns: A ``gettext.NullTranslation`` or ``gettext.GNUTranslation`` with
        fallback languages set.
    """
    try:
        language = gettext.translation("bridgedb", localedir=TRANSLATIONS_DIR,
                                       languages=langs, fallback=True)
        for lang in langs:
            language.add_fallback(
                gettext.translation("bridgedb", localedir=TRANSLATIONS_DIR,
                                    languages=langs, fallback=True))
    except IOError as error:
        logging.error(error.message)

    language.install(unicode=True)
    return language

def usingRTLLang(langs):
    """Check if we should translate the text into a RTL language.

    Choose the first language from the **langs** list that we support and
    return True if it is a RTL language, else return False.

    :param list langs: An incoming request.
    :rtype: bool
    :returns: ``True`` if the preferred language is right-to-left; ``False``
        otherwise.
    """
    lang = getFirstSupportedLang(langs)
    if lang in _langs.RTL_LANGS:
        return True
    return False



"""integration tests for BridgeDB ."""

from __future__ import print_function

import smtplib
import asyncore
import threading
import Queue
import random
import os

from smtpd import SMTPServer

from twisted.trial import unittest
from twisted.trial.unittest import FailTest
from twisted.trial.unittest import SkipTest

from bridgedb.test.util import processExists
from bridgedb.test.util import getBridgeDBPID

# ------------- SMTP Client Config
SMTP_DEBUG_LEVEL = 0  # set to 1 to see SMTP message exchange
BRIDGEDB_SMTP_SERVER_ADDRESS = "localhost"
BRIDGEDB_SMTP_SERVER_PORT = 6725
# %d is parameterised with a random integer to make the sender unique
FROM_ADDRESS_TEMPLATE = "test%d@127.0.0.1"
# Minimum value used to parameterise FROM_ADDRESS_TEMPLATE
MIN_FROM_ADDRESS = 1
# Max value used to parameterise FROM_ADDRESS_TEMPLATE. Needs to be pretty big
# to reduce the chance of collisions
MAX_FROM_ADDRESS = 10**8
TO_ADDRESS = "bridges@torproject.org"
MESSAGE_TEMPLATE = """From: %s
To: %s
Subject: testing

get bridges"""

# ------------- SMTP Server Setup
# Setup an SMTP server which we use to check for responses
# from bridgedb. This needs to be done before sending the actual mail
LOCAL_SMTP_SERVER_ADDRESS = 'localhost'
LOCAL_SMTP_SERVER_PORT = 2525 # Must be the same as bridgedb's EMAIL_SMTP_PORT


class EmailServer(SMTPServer):
    def process_message(self, peer, mailfrom, rcpttos, data):
        ''' Overridden from SMTP server, called whenever a message is received'''
        self.message_queue.put(data)

    def thread_proc(self):
        ''' This function runs in thread, and will continue looping
        until the _stop Event object is set by the stop() function'''
        while self._stop.is_set() == False:
            asyncore.loop(timeout=0.0, count=1)
        # Must close, or asyncore will hold on to the socket and subsequent
        # tests will fail with 'Address not in use'.
        self.close()

    def start(self):
        self.message_queue = Queue.Queue()
        self._stop = threading.Event()
        self._thread = threading.Thread(target=self.thread_proc)
        # Ensures that if any tests do fail, then threads will exit when the
        # parent exits.
        self._thread.setDaemon(True)
        self._thread.start()

    @classmethod
    def startServer(cls):
        #print("Starting SMTP server on %s:%s"
        #      % (LOCAL_SMTP_SERVER_ADDRESS, LOCAL_SMTP_SERVER_PORT))
        server = EmailServer((LOCAL_SMTP_SERVER_ADDRESS,
                              LOCAL_SMTP_SERVER_PORT),
                             None)
        server.start()
        return server

    def stop(self):
        # Signal thread_proc to stop:
        self._stop.set()
        # Wait for thread_proc to return (shouldn't take long)
        self._thread.join()
        assert self._thread.is_alive() == False, "Thread is alive and kicking"

    def getAndCheckMessageContains(self, text, timeoutInSecs=2.0):
        #print("Checking for reponse")
        message = self.message_queue.get(block=True, timeout=timeoutInSecs)
        assert message.find(text) != -1, ("Message did not contain text '%s'."
                                          "Full message is:\n %s"
                                          % (text, message))

    def checkNoMessageReceived(self, timeoutInSecs=2.0):
        try:
            self.message_queue.get(block=True, timeout=timeoutInSecs)
        except Queue.Empty:
            return True
        assert False, "Found a message in the queue, but expected none"

def sendMail(fromAddress):
    #print("Connecting to %s:%d"
    #      % (BRIDGEDB_SMTP_SERVER_ADDRESS, BRIDGEDB_SMTP_SERVER_PORT))
    client = smtplib.SMTP(BRIDGEDB_SMTP_SERVER_ADDRESS,
                          BRIDGEDB_SMTP_SERVER_PORT)
    client.set_debuglevel(SMTP_DEBUG_LEVEL)

    #print("Sending mail TO:%s, FROM:%s"
    #      % (TO_ADDRESS, fromAddress))
    result = client.sendmail(fromAddress, TO_ADDRESS,
                             MESSAGE_TEMPLATE % (fromAddress, TO_ADDRESS))
    assert result == {}, "Failed to send mail"
    client.quit()


class SMTPTests(unittest.TestCase):
    def setUp(self):
        '''Called at the start of each test, ensures that the SMTP server is
        running.
        '''
        here = os.getcwd()
        topdir = here.rstrip('_trial_temp')
        self.rundir = os.path.join(topdir, 'run')
        self.pidfile = os.path.join(self.rundir, 'bridgedb.pid')
        self.pid = getBridgeDBPID(self.pidfile)
        self.server = EmailServer.startServer()

    def tearDown(self):
        '''Called after each test, ensures that the SMTP server is cleaned up.
        '''
        self.server.stop()

    def test_getBridges(self):
        if os.environ.get("CI"):
            if not self.pid or not processExists(self.pid):
                raise FailTest("Could not start BridgeDB process on CI server!")
        if not self.pid or not processExists(self.pid):
            raise SkipTest("Can't run test: no BridgeDB process running.")

        # send the mail to bridgedb, choosing a random email address
        sendMail(fromAddress=FROM_ADDRESS_TEMPLATE
                 % random.randint(MIN_FROM_ADDRESS, MAX_FROM_ADDRESS))

        # then check that our local SMTP server received a response
        # and that response contained some bridges
        self.server.getAndCheckMessageContains("Here are your bridges")

    def test_getBridges_rateLimitExceeded(self):
        if os.environ.get("CI"):
            if not self.pid or not processExists(self.pid):
                raise FailTest("Could not start BridgeDB process on CI server!")
        if not self.pid or not processExists(self.pid):
            raise SkipTest("Can't run test: no BridgeDB process running.")

        # send the mail to bridgedb, choosing a random email address
        FROM_ADDRESS = FROM_ADDRESS_TEMPLATE % random.randint(
            MIN_FROM_ADDRESS, MAX_FROM_ADDRESS)
        sendMail(FROM_ADDRESS)

        # then check that our local SMTP server received a response
        # and that response contained some bridges
        self.server.getAndCheckMessageContains("Here are your bridges")

        # send another request from the same email address
        sendMail(FROM_ADDRESS)

        # this time, the email response should not contain any bridges
        self.server.getAndCheckMessageContains(
            "You have exceeded the rate limit. Please slow down!")

        # then we send another request from the same email address
        sendMail(FROM_ADDRESS)

        # now there should be no response at all (wait 1 second to make sure)
        self.server.checkNoMessageReceived(timeoutInSecs=1.0)

    def test_getBridges_stressTest(self):
        '''Sends a large number of emails in a short period of time, and checks
        that a response is received for each message.
        '''
        if os.environ.get("CI"):
            if not self.pid or not processExists(self.pid):
                raise FailTest("Could not start BridgeDB process on CI server!")
        if not self.pid or not processExists(self.pid):
            raise SkipTest("Can't run test: no BridgeDB process running.")

        NUM_MAILS = 100
        for i in range(NUM_MAILS):
            # Note: if by chance two emails with the same FROM_ADDRESS are
            # generated, this test will fail Setting 'MAX_FROM_ADDRESS' to be
            # a high value reduces the probability of this occuring, but does
            # not rule it out
            sendMail(fromAddress=FROM_ADDRESS_TEMPLATE
                     % random.randint(MIN_FROM_ADDRESS, MAX_FROM_ADDRESS))

        for i in range(NUM_MAILS):
            self.server.getAndCheckMessageContains("Here are your bridges")



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the `bridgedb` commandline script."""

from __future__ import print_function

import os
import signal
import time

from twisted.trial import unittest
from twisted.trial.unittest import FailTest
from twisted.trial.unittest import SkipTest

from bridgedb.test.util import processExists
from bridgedb.test.util import getBridgeDBPID


class BridgeDBCliTest(unittest.TestCase):
    """Test the `bridgedb` command."""

    def setUp(self):
        here = os.getcwd()
        topdir = here.rstrip('_trial_temp')
        self.rundir = os.path.join(topdir, 'run')
        self.pidfile = os.path.join(self.rundir, 'bridgedb.pid')
        self.pid = getBridgeDBPID(self.pidfile)
        self.assignmentsFile = os.path.join(self.rundir, 'assignments.log')

    def doSleep(self):
        """Sleep for some ammount of time.

        We usually have less fake bridge descriptors with CI runs than we do
        during other tests, so we can safely decrease the sleep time on CI
        machines.
        """
        if os.environ.get("TRAVIS"):
            time.sleep(10)
        else:
            time.sleep(20)
        return

    def test_bridgedb_assignments_log(self):
        """This test should only be run if a BridgeDB server has already been
        started in another process.

        To see how this is done for the Travis CI tests, see the
        'before_script' section of the ``.travis.yml`` file in the top
        directory of this repository.

        This test ensures that an ``assignments.log`` file is created after a
        BridgeDB process was started.
        """
        if os.environ.get("CI"):
            if not self.pid or not processExists(self.pid):
                raise FailTest("Could not start BridgeDB process on CI server!")
        if not self.pid or not processExists(self.pid):
            raise SkipTest("Can't run test: no BridgeDB process running.")

        self.assertTrue(os.path.isfile(self.assignmentsFile))

    def test_bridgedb_SIGHUP_assignments_log(self):
        """Test that BridgeDB creates a new ``assignments.log`` file after
        receiving a SIGHUP.
        """
        if os.environ.get("CI"):
            if not self.pid or not processExists(self.pid):
                raise FailTest("Could not start BridgeDB process on CI server!")
        if not self.pid or not processExists(self.pid):
            raise SkipTest("Can't run test: no BridgeDB process running.")

        os.unlink(self.assignmentsFile)
        os.kill(self.pid, signal.SIGHUP)
        self.doSleep()
        self.assertTrue(os.path.isfile(self.assignmentsFile))

    def test_bridgedb_SIGUSR1_buckets(self):
        """Test that BridgeDB dumps buckets appropriately after a SIGUSR1."""
        if os.environ.get("CI"):
            if not self.pid or not processExists(self.pid):
                raise FailTest("Could not start BridgeDB process on CI server!")
        if not self.pid or not processExists(self.pid):
            raise SkipTest("Can't run test: no BridgeDB process running.")

        os.kill(self.pid, signal.SIGUSR1)
        self.doSleep()
        buckets = [['email', False], ['https', False], ['unallocated', False]]
        for rundirfile in os.listdir(self.rundir):
            for bucket in buckets:
                if rundirfile.startswith(bucket[0]):
                    bucket[1] = True
                    break
        for bucket in buckets:
            self.assertTrue(bucket[1], "%s bucket was not dumped!" % bucket[0])



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests utilitys the `bridgedb.test` package."""

from __future__ import print_function
from __future__ import unicode_literals

import errno
import os

from functools import wraps

from twisted.trial import unittest

from bridgedb import util as bdbutil


def fileCheckDecorator(func):
    """Method decorator for a t.t.unittest.TestCase test_* method.

    .. codeblock:: python

        import shutil
        from twisted.trial import unittest

        pyunit = __import__('unittest')

        class TestTests(unittest.TestCase):
            @fileCheckDecorator
            def doCopyFile(src, dst, description=None):
                shutil.copy(src, dst)
            def test_doCopyFile(self):
                srcfile = self.mktemp()
                dstfile = self.mktemp()
                with open(srcfile, 'wb') as fh:
                    fh.write('testing TestCase method decorator utility')
                    fh.flush()
                self.doCopyFile(srcfile, dstfile, 'asparagus')

        testtest = TestTests()
        testtest.runTest()

    ..

    :type func: callable
    :param func: The ``test_*`` method, from a
                 :api:`twisted.trial.unittest.TestCase` instance, to wrap.
    """
    @wraps(func)
    def wrapper(self, src, dst, description):
        self.assertTrue(os.path.isfile(src),
                        "Couldn't find original %s file: %r"
                        % (str(description), src))
        func(self, src, dst, description)
        self.assertTrue(os.path.isfile(dst),
                        "Couldn't find new %s file: %r. Original: %r"
                        % (str(description), dst, src))
    return wrapper

def processExists(pid):
    """Test if the process with **pid** exists.

    :param int pid: An integer specifying the process ID.
    :raises: OSError, if ``OSError.errno`` wasn't an expected errno (according
        to the "ERRORS" section from ``man 2 kill``).
    :rtype: bool
    :returns: ``True`` if a process with **pid** exists, ``False`` otherwise.
    """
    try:
        os.kill(pid, 0)
    except OSError as err:
        if err.errno == errno.ESRCH:  # ESRCH: No such process
            return False
        if err.errno == errno.EPERM:  # EPERM: Operation not permitted
            # If we're not allowed to signal the process, then there exists a
            # process that we don't have permissions to access.
            return True
        else:
            raise
    else:
        return True

def getBridgeDBPID(pidfile="bridgedb.pid"):
    """Read the ``bridgedb.pid`` file in **rundir**, if it exists, to get the
    PID.

    :param str pidfile: The path to the BridgeDB pidfile.
    :rtype: int
    :returns: The process ID, if available, otherwise ``0``.
    """
    fh = None
    try:
        fh = open(pidfile)
    except (IOError, OSError) as err:
        print(err)
        pid = 0
    else:
        pid = int(fh.read())

    if fh:
        fh.close()

    return pid


#: Mixin class for use with :api:`~twisted.trial.unittest.TestCase`. A
#: ``TestCaseMixin`` can be used to add additional methods, which should be
#: common to multiple ``TestCase`` subclasses, without the ``TestCaseMixin``
#: being run as a ``TestCase`` by ``twisted.trial``.
TestCaseMixin = bdbutil.mixin
TestCaseMixin.register(unittest.TestCase)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.bridges` module."""

from binascii import a2b_hex

import datetime
import ipaddr
import io
import hashlib
import os
import warnings

from twisted.trial import unittest

from bridgedb import bridges
from bridgedb.Bridges import FilteredBridgeSplitter
from bridgedb.bridgerequest import BridgeRequestBase
from bridgedb.parse import descriptors
from bridgedb.parse.addr import PortList
from bridgedb.parse.nickname import InvalidRouterNickname


# Don't print "WARNING:root: Couldn't parse K=V from PT arg: ''" a bunch of
# times while running the tests.
warnings.filterwarnings("ignore", ".*Couldn't parse K=V from PT arg.*", Warning)


BRIDGE_NETWORKSTATUS = '''\
r FourfoldQuirked LDIlxIBTMQJeIR9Lblv0XDM/3Sw c4EVu2rO/iD/DJYBX/Ll38DGQWI 2014-12-22 21:51:27 179.178.155.140 36489 0
a [6bf3:806b:78cd:d4b4:f6a7:4ced:cfad:dad4]:36488
s Fast Guard Running Stable Valid
w Bandwidth=1585163
p reject 1-65535
'''

BRIDGE_SERVER_DESCRIPTOR = '''\
@purpose bridge
router FourfoldQuirked 179.178.155.140 36489 0 0
or-address [6bf3:806b:78cd:d4b4:f6a7:4ced:cfad:dad4]:36488
platform Tor 0.2.3.24-rc on Linux
opt protocols Link 1 2 Circuit 1
published 2014-12-22 21:51:27
opt fingerprint 2C32 25C4 8053 3102 5E21 1F4B 6E5B F45C 333F DD2C
uptime 33200687
bandwidth 1866688205 2110169275 1623207134
opt extra-info-digest 4497E81715D958105C6A39D348163AD8F3080FB2
onion-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBANKicvIGWp9WGKOJV8Fs3YKdTDrgxlggzyKgkW+MZWEPQ9lLcrmXqBdW
nVK5EABByHnnxJfk+sm+6yDYxY/lFVL1SEP84pAK1Z21f4+grNlwox1DLyntXDdz
BCZuRszuBYK3ncsk+ePQeUzRKQ/GZt9s/oy0IjtNbAoAoq7DKUVzAgMBAAE=
-----END RSA PUBLIC KEY-----
signing-key
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBALnJK7A9aZIp2ry9ruVYzm4VfaXzNHdTcvkXTrETu/jLXsosEwj9viSe
Ry3W/uctbjzdwlIY0ZBUuV20q9bh+/c7Q0T8LOHBZouOy+nhFOUX+Q5YCG9cRnY0
hBebYTzyplh0tT8xyYwcS8y6esL+gjVDLo6Og3QPhWRFQ4CyCic9AgMBAAE=
-----END RSA PUBLIC KEY-----
contact Somebody <somebody@example.com>
ntor-onion-key aVmfOm9C046wM8ktGnpfBHSNj1Jm30M/m2P7W3a7Xn8
reject *:*
router-signature
-----BEGIN SIGNATURE-----
nxml4rTyTrj8dHcsFt2B4ACz2AN5CuZ2t5UF1BtXUpuzHmqVlg7imy8Cp2xIwoDa
4uv/tTG32macauVnMHt0hSbtBF5nHfxU9G1T/XzdtL+KD8REDGky4allXnmvF6In
rFtSn2OeZewvi8oYPmVYKgzHL6tzZxs2Sn/bOTj5sRw=
-----END SIGNATURE-----
'''

BRIDGE_EXTRAINFO = '''\
extra-info FourfoldQuirked 2C3225C4805331025E211F4B6E5BF45C333FDD2C
published 2014-12-22 21:51:27
write-history 2014-12-22 21:51:27 (900 s) 3188736,2226176,2866176
read-history 2014-12-22 21:51:27 (900 s) 3891200,2483200,2698240
dirreq-write-history 2014-12-22 21:51:27 (900 s) 1024,0,2048
dirreq-read-history 2014-12-22 21:51:27 (900 s) 0,0,0
geoip-db-digest 51AE9611B53880B2BCF9C71E735D73F33FAD2DFE
geoip6-db-digest 26B0D55B20BEB496A3ADE7C6FDD866F5A81027F7
dirreq-stats-end 2014-12-22 21:51:27 (86400 s)
dirreq-v3-ips
dirreq-v3-reqs
dirreq-v3-resp ok=16,not-enough-sigs=0,unavailable=0,not-found=0,not-modified=0,busy=0
dirreq-v3-direct-dl complete=0,timeout=0,running=0
dirreq-v3-tunneled-dl complete=12,timeout=0,running=0
transport obfs3 179.178.155.140:36490
transport obfs2 179.178.155.140:36491
transport scramblesuit 179.178.155.140:36492 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
transport obfs4 179.178.155.140:36493 iat-mode=0,node-id=25293f2761d658cc70c19515861842d712751bdc,public-key=02d20bbd7e394ad5999a4cebabac9619732c343a4cac99470c03e23ba2bdc2bc
bridge-stats-end 2014-12-22 21:51:27 (86400 s)
bridge-ips ca=8
bridge-ip-versions v4=8,v6=0
bridge-ip-transports <OR>=8
router-signature
-----BEGIN SIGNATURE-----
cn4+8pQwCMPnHcp1s8wm7ZYsnd9AXJH6ysNlvQ63jsPCG9JdE5E8BwCThEgUccJI
XILT4o+SveEQUG72R4bENsKxqV4rRNh1g6CNAbYhAITqrU9B+jImDgrBBW+XWT5K
78ECRPn6Y4KsxFb0TIn7ddv9QjApyBJNIDMihH80Yng=
-----END SIGNATURE-----
'''


class BridgeIntegrationTests(unittest.TestCase):
    """Integration tests to ensure that the new :class:`bridgedb.bridges.Bridge`
    class has compatible behaviour with the expected behaviour of the old
    :class:`bridgedb.Bridges.Bridge` class.

    .. data: OldTest (enum)

    These tests were refactored from the old tests for
    :class:`~bridgedb.test.deprecated.Bridge`, which lived in
    ``lib/bridgedb/test/test_Bridges.py``. For the translations from the old
    tests in ``bridgedb.test.test_Bridges.BridgeClassTest`` to their new
    equivalents here in ``bridgedb.test.test_bridges.BridgeIntegrationTests``,
    which should test for the same things as their old equivalents, see the
    following table:

    ==============================================  ========================
    OldTest                                         Equivalent Test(s) Here
    ==============================================  ========================
    test_init                                       test_integration_init_[0-5]
    test_getID                                      test_integration_getID
    test_setDescriptorDigest                        test_integration_setDescriptorDigest
    test_setExtraInfoDigest                         test_integration_setExtraInfoDigest
    test_setVerified                                test_integration_setVerified
    test_setRunningStable                           test_integration_setRunningStable
    test_getConfigLine_vanilla_withoutFingerprint   test_integration_getConfigLine_vanilla_withoutFingerprint
    test_getConfigLine_vanilla_withFingerprint      test_integration_getConfigLine_vanilla_withFingerprint
    test_getConfigLine_scramblesuit_withFingeprint  test_integration_getConfigLine_scramblesuit_withFingerprint
    test_splitterBridgeInsertion                    test_integration_splitterBridgeInsertion
    ==============================================  ========================
    ..
    """

    def setUp(self):
        self.nickname = 'unnamed'
        self.ip = ipaddr.IPAddress('127.0.0.1')
        self.orport = '9001'
        self.fingerprint = 'A1CC8DFEF1FA11AF9C40AF1054DF9DAF45250556'
        self.id_digest = a2b_hex(self.fingerprint)
        self.or_addresses = {ipaddr.IPAddress('6.6.6.6'): PortList(6666),
                             ipaddr.IPAddress('42.1.42.1'): PortList(443)}

    def test_integration_init_0(self):
        """Ensure that we can initialise the new :class:`bridgedb.bridges.Bridge`
        class in the same manner as the old :class:`bridgedb.Bridges.Bridge`
        class. This test ensures that initialisation with a fingerprint is
        successful.
        """
        b = bridges.Bridge(self.nickname, self.ip, self.orport,
                           fingerprint=self.fingerprint)
        self.assertIsInstance(b, bridges.Bridge)

    def test_integration_init_1(self):
        """Ensure that we can initialise the new :class:`bridgedb.bridges.Bridge`
        class in the same manner as the old :class:`bridgedb.Bridges.Bridge`
        class. This test ensures that initialisation with a digest of a
        bridge's ID key is successful.
        """
        b = bridges.Bridge(self.nickname, self.ip, self.orport,
                           id_digest=self.id_digest)
        self.assertIsInstance(b, bridges.Bridge)

    def test_integration_init_2(self):
        """Initialisation of a :class:`bridgedb.bridges.Bridge` with a bad
        ``id_digest`` should raise a TypeError.
        """
        self.failUnlessRaises(TypeError, bridges.Bridge,
                              self.nickname, self.ip, self.orport,
                              id_digest=self.id_digest[:-1])

    def test_integration_init_3(self):
        """Initialisation of a :class:`bridgedb.bridges.Bridge` with a bad
        ``fingerprint`` should raise a TypeError.
        """
        self.failUnlessRaises(TypeError, bridges.Bridge,
                              self.nickname, self.ip, self.orport,
                              fingerprint=self.fingerprint[:-1])

    def test_integration_init_4(self):
        """Initialisation of a :class:`bridgedb.bridges.Bridge` with a bad
        ``fingerprint`` should raise a TypeError.
        """
        invalid_fingerprint = self.fingerprint[:-1] + 'q'
        self.failUnlessRaises(TypeError, bridges.Bridge, self.nickname,
                              self.ip, self.orport,
                              fingerprint=invalid_fingerprint)

    def test_integration_init_5(self):
        """Initialisation of a :class:`bridgedb.bridges.Bridge` without either
        a ``fingerprint`` or an ``id_digest`` should raise a TypeError.
        """
        self.failUnlessRaises(TypeError, bridges.Bridge,
                              self.nickname, self.ip, self.orport)

    def test_integration_getID(self):
        """Calling ``bridges.Bridge.getID()`` should return the binary encoded
        ``fingerprint``.
        """
        bridge = bridges.Bridge(self.nickname, self.ip, self.orport,
                                self.fingerprint)
        self.assertEqual(self.id_digest, bridge.getID())

    def test_integration_setDescriptorDigest(self):
        """Test setting the server-descriptor digest value."""
        bridge = bridges.Bridge(self.nickname, self.ip, self.orport,
                                self.fingerprint)
        testtext = 'thisisatest'
        bridge.setDescriptorDigest(testtext)
        self.assertEqual(bridge.desc_digest, testtext)

    def test_integration_setExtraInfoDigest(self):
        """Test setting the extra-info digest value."""
        bridge = bridges.Bridge(self.nickname, self.ip, self.orport,
                                self.fingerprint)
        testtext = 'thisisatest'
        bridge.setExtraInfoDigest(testtext)
        self.assertEqual(bridge.ei_digest, testtext)

    def test_integration_setVerified(self):
        """Test setting the `verified` attribute on a Bridge."""
        raise unittest.SkipTest(
            ("The setVerified() and isVerified() methods were not refactored "
             "into the new bridgedb.bridges.Bridge class, as it's not clear "
             "yet if they are necessary. Skip these tests for now."))

        bridge = bridges.Bridge(self.nickname, self.ip, self.orport,
                                self.fingerprint)
        bridge.setVerified()
        self.assertTrue(bridge.isVerified())
        self.assertTrue(bridge.verified)
        self.assertEqual(self.id_digest, bridge.getID())

    def test_integration_setRunningStable(self):
        """Test setting the `running` and `stable` attributes on a Bridge."""
        bridge = bridges.Bridge(self.nickname, self.ip, self.orport,
                                self.fingerprint)
        self.assertFalse(bridge.running)
        self.assertFalse(bridge.stable)
        bridge.setStatus(True, True)
        self.assertTrue(bridge.running)
        self.assertTrue(bridge.stable)

    def test_integration_getConfigLine_vanilla_withoutFingerprint(self):
        """Should return a config line without a fingerprint."""
        #self.skip = True
        bridge = bridges.Bridge('nofpr', '23.23.23.23', 2323, self.fingerprint,
                                or_addresses=self.or_addresses)
        bridgeLine = bridge.getConfigLine()
        ip = bridgeLine.split(':')[0]
        self.assertTrue(ipaddr.IPAddress(ip))

    def test_integration_getConfigLine_vanilla_withFingerprint(self):
        """Should return a config line with a fingerprint."""
        bridge = bridges.Bridge('fpr', '23.23.23.23', 2323,
                                id_digest=self.id_digest,
                                or_addresses=self.or_addresses)
        bridgeLine = bridge.getConfigLine(includeFingerprint=True)
        self.assertIsNotNone(bridgeLine)
        self.assertSubstring(self.fingerprint, bridgeLine)
        ip = bridgeLine.split(':')[0]
        self.assertTrue(ipaddr.IPAddress(ip))

    def test_integration_getConfigLine_scramblesuit_withFingerprint(self):
        """Should return a scramblesuit config line with a fingerprint."""
        bridge = bridges.Bridge('philipkdick', '23.23.23.23', 2323,
                                id_digest=self.id_digest,
                                or_addresses=self.or_addresses)
        ptArgs = {'password': 'NEQGQYLUMUQGK5TFOJ4XI2DJNZTS4LRO'}
        pt = bridges.PluggableTransport(bridge.fingerprint, 'scramblesuit',
                                        ipaddr.IPAddress('42.42.42.42'), 4242,
                                        ptArgs)
        bridge.transports.append(pt)
        bridgeLine = bridge.getConfigLine(includeFingerprint=True,
                                          transport='scramblesuit')
        ptArgsList = ' '.join(["{0}={1}".format(k,v) for k,v in ptArgs.items()])
        self.assertEqual("scramblesuit 42.42.42.42:4242 %s %s"
                         % (self.fingerprint, ptArgsList),
                         bridgeLine)

    def test_integration_splitterBridgeInsertion(self):
        key = "Testing-Bridges-To-Rings"
        splitter = FilteredBridgeSplitter(key)

        bridge1 = bridges.Bridge('unamed1', '1.2.3.5', 9100,
                                 'a1cc8dfef1fa11af9c40af1054df9daf45250550')
        bridge1.setStatus(running = True)
        bridge2 = bridges.Bridge('unamed2', '1.2.3.4', 8080,
                                 'a1cc8dfef1fa11af9c40af1054df9daf45250551')
        bridge2.setStatus(running = True)
        bridge3 = bridges.Bridge('unamed3', '5.2.3.4', 8080,
                                 'b1cc8dfef1fa11af9c40af1054df9daf45250552')
        bridge3.setStatus(running = True)
        bridge4 = bridges.Bridge('unamed3', '5.2.3.4', 8080,
                                 'b1cc8dfef1fa11af9c40af1054df9daf45250552')
        bridge4.setStatus(running = True)

        self.failUnlessEqual(len(splitter), 0)
        splitter.insert(bridge1)
        splitter.insert(bridge2)
        splitter.insert(bridge3)
        # Check that all were inserted
        self.failUnlessEqual(len(splitter), 3)
        splitter.insert(bridge1)
        # Check that the same bridge is not inserted twice
        self.failUnlessEqual(len(splitter), 3)
        splitter.insert(bridge4)
        # Check that identical bridges are not inserted twice
        self.failUnlessEqual(len(splitter), 3)


class FlagsTests(unittest.TestCase):
    """Tests for :class:`bridgedb.bridges.Flags`."""

    def setUp(self):
        self.flags = bridges.Flags()
        self._all_flag_names = ["fast", "guard", "running", "stable", "valid"]

    def test_init(self):
        """Upon initialisation, all flags should be ``False``."""
        for flag in self._all_flag_names:
            f = getattr(self.flags, flag, None)
            self.assertFalse(f, "%s should be False" % flag)

    def test_settingStable(self):
        """Setting the Stable flag to ``True`` should result in Flags.stable
        being ``True``.
        """
        self.flags.stable = True
        self.assertTrue(self.flags.stable, "The Stable flag should be True")

    def test_settingRunning(self):
        """Setting the Running flag to ``True`` should result in Flags.running
        being ``True``.
        """
        self.flags.running = True
        self.assertTrue(self.flags.running, "The Running flag should be True")

    def test_changingFlags(self):
        """Setting a flag and then unsetting it should result in it being
        ``True`` and then ``False``.
        """
        self.flags.valid = True
        self.assertTrue(self.flags.valid, "The Valid flag should be True")
        self.flags.valid = False
        self.assertFalse(self.flags.valid, "The Valid flag should be False")

    def test_update_Fast_Stable(self):
        """Test changing flags with the update() method."""
        self.flags.update(["Fast", "Stable"])
        self.assertTrue(self.flags.fast)
        self.assertTrue(self.flags.stable)

    def test_update_Fast(self):
        """Test changing flags with the update() method."""
        self.flags.update(["Fast"])
        self.assertTrue(self.flags.fast)
        self.assertFalse(self.flags.stable)

    def test_update_Stable(self):
        """Test changing flags with the update() method."""
        self.flags.update(["Stable"])
        self.assertFalse(self.flags.fast)
        self.assertTrue(self.flags.stable)


class BridgeAddressBaseTests(unittest.TestCase):
    """Tests for :class:`bridgedb.bridges.BridgeAddressBase`."""

    def setUp(self):
        self.fingerprint = '2C3225C4805331025E211F4B6E5BF45C333FDD2C'
        self.bab = bridges.BridgeAddressBase()

    def test_BridgeAddressBase_init(self):
        """The BridgeAddressBase's _address and _fingerprint should be None."""
        self.assertIsNone(self.bab._address)
        self.assertIsNone(self.bab._fingerprint)

    def test_BridgeAddressBase_fingerprint_del(self):
        """The del method for the fingerprint property should reset the
        fingerprint to None.
        """
        self.bab.fingerprint = self.fingerprint
        self.assertEqual(self.bab.fingerprint, self.fingerprint)

        del(self.bab.fingerprint)
        self.assertIsNone(self.bab.fingerprint)
        self.assertIsNone(self.bab._fingerprint)

    def test_BridgeAddressBase_address_del(self):
        """The del method for the address property should reset the
        address to None.
        """
        self.bab.address = '11.12.13.14'
        self.assertEqual(self.bab.address, ipaddr.IPv4Address('11.12.13.14'))

        del(self.bab.address)
        self.assertIsNone(self.bab.address)
        self.assertIsNone(self.bab._address)

    def test_BridgeAddressBase_country(self):
        """The getter method for the country property should get the
        address's geoIP country code.
        """
        self.bab.address = '11.12.13.14'
        self.assertEqual(self.bab.address, ipaddr.IPv4Address('11.12.13.14'))

        cc = self.bab.country
        self.assertIsNotNone(cc)
        self.assertIsInstance(cc, basestring)
        self.assertEqual(len(cc), 2)


class PluggableTransportTests(unittest.TestCase):
    """Tests for :class:`bridgedb.bridges.PluggableTransport."""

    def setUp(self):
        self.fingerprint = "ABCDEF0123456789ABCDEF0123456789ABCDEF01"

    def test_PluggableTransport_init_with_parameters(self):
        """Initialising a PluggableTransport with args should work."""
        pt = bridges.PluggableTransport(self.fingerprint,
                                        "voltronPT", "1.2.3.4", 443,
                                        {'sharedsecret': 'foobar'})
        self.assertIsInstance(pt, bridges.PluggableTransport)

    def test_PluggableTransport_init(self):
        """Initialising a PluggableTransport without args should work."""
        pt = bridges.PluggableTransport()
        self.assertIsInstance(pt, bridges.PluggableTransport)

    def test_PluggableTransport_port_del(self):
        """The del method for the port property should reset the port to None.
        """
        pt = bridges.PluggableTransport(self.fingerprint,
                                        "voltronPT", "1.2.3.4", 443,
                                        {'sharedsecret': 'foobar'})
        self.assertEqual(pt.port, 443)

        del(pt.port)
        self.assertIsNone(pt.port)
        self.assertIsNone(pt._port)

    def test_PluggableTransport_parseArgumentsIntoDict_valid_list(self):
        """Parsing a valid list of PT args should return a dictionary."""
        pt = bridges.PluggableTransport()
        args = pt._parseArgumentsIntoDict(["sharedsecret=foobar",
                                           "publickey=1234"])
        self.assertIsInstance(args, dict)
        self.assertItemsEqual(args, {"sharedsecret": "foobar",
                                     "publickey": "1234"})

    def test_PluggableTransport_parseArgumentsIntoDict_valid_list_multi(self):
        """Parsing a valid list with multiple PT args in a single list element
        should return a dictionary.
        """
        pt = bridges.PluggableTransport()
        args = pt._parseArgumentsIntoDict(["sharedsecret=foobar,password=baz",
                                           "publickey=1234"])
        self.assertIsInstance(args, dict)
        self.assertItemsEqual(args, {"sharedsecret": "foobar",
                                     "password": "baz",
                                     "publickey": "1234"})

    def test_PluggableTransport_parseArgumentsIntoDict_invalid_missing_equals(self):
        """Parsing a string of PT args where one PT arg (K=V) is missing an
        ``=`` character should raise a ValueError.
        """
        pt = bridges.PluggableTransport()
        args = pt._parseArgumentsIntoDict(
            ["sharedsecret=foobar,password,publickey=1234"])
        self.assertItemsEqual(args, {"sharedsecret": "foobar",
                                     "publickey": "1234"})

    def test_PluggableTransport_runChecks_invalid_fingerprint(self):
        """Calling _runChecks() on a PluggableTransport with an invalid
        fingerprint should raise a MalformedPluggableTransport exception.
        """
        pt = bridges.PluggableTransport()
        self.assertRaises(
            bridges.MalformedPluggableTransport,
            pt.updateFromStemTransport,
            "INVALIDFINGERPRINT", 'obfs4', ('34.230.223.87', 37341, [
                ('iat-mode=0,'
                 'node-id=2a79f14120945873482b7823caabe2fcde848722,'
                 'public-key=0a5b046d07f6f971b7776de682f57c5b9cdc8fa060db7ef59de82e721c8098f4')]))

    def test_PluggableTransport_runChecks_invalid_ip(self):
        """Calling _runChecks() on a PluggableTransport with an invalid
        IP address should raise a InvalidPluggableTransportIP exception.
        """
        pt = bridges.PluggableTransport()
        self.assertRaises(
            bridges.InvalidPluggableTransportIP,
            pt.updateFromStemTransport,
            self.fingerprint, 'obfs4', ('34.230.223', 37341, [
                ('iat-mode=0,'
                 'node-id=2a79f14120945873482b7823caabe2fcde848722,')]))

    def test_PluggableTransport_runChecks_invalid_port_type(self):
        """Calling _runChecks() on a PluggableTransport with an invalid port
        should raise a MalformedPluggableTransport exception.
        """
        pt = bridges.PluggableTransport()
        self.assertRaises(
            bridges.MalformedPluggableTransport,
            pt.updateFromStemTransport,
            self.fingerprint, 'obfs4', ('34.230.223.87', "anyport", [
                ('iat-mode=0,'
                 'node-id=2a79f14120945873482b7823caabe2fcde848722,')]))

    def test_PluggableTransport_runChecks_invalid_port_range(self):
        """Calling _runChecks() on a PluggableTransport with an invalid port
        (too high) should raise a MalformedPluggableTransport exception.
        """
        pt = bridges.PluggableTransport()
        self.assertRaises(
            bridges.MalformedPluggableTransport,
            pt.updateFromStemTransport,
            self.fingerprint, 'obfs4', ('34.230.223.87', 65536, [
                ('iat-mode=0,'
                 'node-id=2a79f14120945873482b7823caabe2fcde848722,')]))

    def test_PluggableTransport_runChecks_invalid_pt_args(self):
        """Calling _runChecks() on a PluggableTransport with an invalid PT
        args should raise a MalformedPluggableTransport exception.
        """
        try:
            pt = bridges.PluggableTransport(self.fingerprint,
                                            "voltronPT", "1.2.3.4", 443,
                                            'sharedsecret=foobar')
        except Exception as error:
            self.failUnlessIsInstance(error,
                                      bridges.MalformedPluggableTransport)

    def test_PluggableTransport_getTransportLine_bridge_prefix(self):
        """If the 'Bridge ' prefix was requested, then it should be at the
        beginning of the bridge line.
        """
        pt = bridges.PluggableTransport(self.fingerprint,
                                        "voltronPT", "1.2.3.4", 443,
                                        {'sharedsecret': 'foobar',
                                         'password': 'unicorns'})
        bridgeLine = pt.getTransportLine(bridgePrefix=True)
        self.assertTrue(bridgeLine.startswith("Bridge "))

    def test_PluggableTransport_getTransportLine_without_Fingerprint(self):
        """If no fingerprint was requested, then there shouldn't be a
        fingerprint in the bridge line.
        """
        pt = bridges.PluggableTransport(self.fingerprint,
                                        "voltronPT", "1.2.3.4", 443,
                                        {'sharedsecret': 'foobar',
                                         'password': 'unicorns'})
        bridgeLine = pt.getTransportLine(includeFingerprint=False)
        self.assertNotSubstring(self.fingerprint, bridgeLine)

    def test_PluggableTransport_getTransportLine_content_order(self):
        """Check the order and content of the bridge line string."""
        pt = bridges.PluggableTransport(self.fingerprint,
                                        "voltronPT", "1.2.3.4", 443,
                                        {'sharedsecret': 'foobar',
                                         'password': 'unicorns'})
        bridgeLine = pt.getTransportLine()

        # We have to check for substrings because we don't know which order
        # the PT arguments will end up in the bridge line.  Fortunately, the
        # following three are the only ones which are important to have in
        # order:
        self.assertTrue(bridgeLine.startswith("voltronPT"))
        self.assertSubstring("voltronPT 1.2.3.4:443 " + self.fingerprint,
                             bridgeLine)
        # These ones can be in any order, but they should be at the end of the
        # bridge line:
        self.assertSubstring("password=unicorns", bridgeLine)
        self.assertSubstring("sharedsecret=foobar", bridgeLine)

    def test_PluggableTransport_getTransportLine_ptargs_space_delimited(self):
        """The PT arguments in a bridge line should be space-separated."""
        pt = bridges.PluggableTransport(self.fingerprint,
                                        "voltronPT", "1.2.3.4", 443,
                                        {'sharedsecret': 'foobar',
                                         'password': 'unicorns'})
        bridgeLine = pt.getTransportLine()
        self.assertTrue(
            ("password=unicorns sharedsecret=foobar" in bridgeLine) or
            ("sharedsecret=foobar password=unicorns" in bridgeLine))

    def test_PluggableTransport_getTransportLine_IPv6(self):
        """The address portion of a bridge line with an IPv6 address should
        have square brackets around it.
        """
        pt = bridges.PluggableTransport(self.fingerprint,
                                        "voltronPT", "2006:42::1234", 443,
                                        {'sharedsecret': 'foobar',
                                         'password': 'unicorns'})
        bridgeLine = pt.getTransportLine()
        self.assertEqual(pt.address.version, 6)
        self.assertIn("[2006:42::1234]:443", bridgeLine)


class BridgeBackwardsCompatibilityTests(unittest.TestCase):
    """Tests for :class:`bridgedb.bridges.BridgeBackwardsCompatibility`."""

    def setUp(self):
        self.nickname = "RouterNickname"
        self.address = "23.23.23.23"
        self.orPort = 9001
        self.fingerprint = "0123456789ABCDEF0123456789ABCDEF01234567"
        self.orAddresses = {"2006:42::123F": PortList(443, 9001, 1337),
                            "2006:42::123E": PortList(9001, 993)}

    def test_BridgeBackwardsCompatibility_init_with_PortList(self):
        """Test initialisation with the usual number of valid arguments and
        PortLists for the orAddresses.
        """
        bridge = bridges.BridgeBackwardsCompatibility(
            self.nickname,
            self.address,
            self.orPort,
            self.fingerprint,
            self.orAddresses)
        self.assertIsInstance(bridge, bridges.BridgeBackwardsCompatibility)

    def test_BridgeBackwardsCompatibility_init_without_PortList(self):
        """Test initialisation with the usual number of valid arguments and
        integers for the orAddresses' ports.
        """
        bridge = bridges.BridgeBackwardsCompatibility(
            self.nickname,
            self.address,
            self.orPort,
            self.fingerprint,
            {"2006:42::123F": 443,
             "2006:42::123E": 9001})
        self.assertIsInstance(bridge, bridges.BridgeBackwardsCompatibility)

    def test_BridgeBackwardsCompatibility_init_without_address(self):
        """Test initialisation without an IP address."""
        bridge = bridges.BridgeBackwardsCompatibility(
            nickname=self.nickname,
            orport=self.orPort,
            fingerprint=self.fingerprint,
            or_addresses={"2006:42::123F": 443, "2006:42::123E": 9001})
        self.assertIsInstance(bridge, bridges.BridgeBackwardsCompatibility)

    def test_BridgeBackwardsCompatibility_init_invalid_orAddresses_address(self):
        """Test initialisation with an invalid ORAddress."""
        bridge = bridges.BridgeBackwardsCompatibility(
            nickname=self.nickname,
            ip=self.address,
            orport=self.orPort,
            fingerprint=self.fingerprint,
            or_addresses={"10.1.2.3": 443, "2006:42::123E": 9001})
        self.assertIsInstance(bridge, bridges.BridgeBackwardsCompatibility)
        self.assertEqual(len(bridge.orAddresses), 1)

    def test_BridgeBackwardsCompatibility_init_invalid_orAddresses_port(self):
        """Test initialisation with an invalid ORPort."""
        bridge = bridges.BridgeBackwardsCompatibility(
            nickname=self.nickname,
            ip=self.address,
            orport=self.orPort,
            fingerprint=self.fingerprint,
            or_addresses={"2006:42::123F": 443, "2006:42::123E": "anyport"})
        self.assertIsInstance(bridge, bridges.BridgeBackwardsCompatibility)
        self.assertEqual(len(bridge.orAddresses), 1)

    def test_BridgeBackwardsCompatibility_setStatus_running(self):
        """Using setStatus() to set the Running flag should set Bridge.running
        and Bridge.flags.running to True.
        """
        bridge = bridges.BridgeBackwardsCompatibility(
            nickname=self.nickname,
            ip=self.address,
            orport="anyport",
            fingerprint=self.fingerprint,
            or_addresses={"2006:42::123F": 443, "2006:42::123E": 9001})
        self.assertIsInstance(bridge, bridges.BridgeBackwardsCompatibility)
        self.assertFalse(bridge.running)
        self.assertFalse(bridge.flags.running)

        bridge.setStatus(running=True)
        self.assertTrue(bridge.running)
        self.assertTrue(bridge.flags.running)

    def test_BridgeBackwardsCompatibility_setStatus_running(self):
        """Using setStatus() to set the Running and Stable flags should set
        Bridge.running, Bridge.flags.running, Bridge.stable, and
        Bridge.flags.stable.
        """
        bridge = bridges.BridgeBackwardsCompatibility(
            nickname=self.nickname,
            ip=self.address,
            orport="anyport",
            fingerprint=self.fingerprint,
            or_addresses={"2006:42::123F": 443, "2006:42::123E": 9001})
        self.assertIsInstance(bridge, bridges.BridgeBackwardsCompatibility)
        self.assertFalse(bridge.running)
        self.assertFalse(bridge.flags.running)
        self.assertFalse(bridge.stable)
        self.assertFalse(bridge.flags.stable)

        bridge.setStatus(running=True, stable=True)
        self.assertTrue(bridge.running)
        self.assertTrue(bridge.flags.running)
        self.assertTrue(bridge.stable)
        self.assertTrue(bridge.flags.stable)


class BridgeTests(unittest.TestCase):
    """Tests for :class:`bridgedb.bridges.Bridge`."""

    def _parseAllDescriptorFiles(self):
        self.networkstatus = descriptors.parseNetworkStatusFile(
            self._networkstatusFile)[0]
        self.serverdescriptor = descriptors.parseServerDescriptorsFile(
            self._serverDescriptorFile)[0]
        self.extrainfo = descriptors.parseExtraInfoFiles(
            self._extrainfoFile).values()[0]

    def _writeNetworkstatus(self, networkstatus):
        with open(self._networkstatusFile, 'w') as fh:
            fh.write(networkstatus)
            fh.flush()

    def _writeServerdesc(self, serverdesc):
        with open(self._serverDescriptorFile, 'w') as fh:
            fh.write(serverdesc)
            fh.flush()

    def _writeExtrainfo(self, extrainfo):
        with open(self._extrainfoFile, 'w') as fh:
            fh.write(extrainfo)
            fh.flush()

    def _writeDescriptorFiles(self, networkstatus, serverdesc, extrainfo):
        self._writeNetworkstatus(networkstatus)
        self._writeServerdesc(serverdesc)
        self._writeExtrainfo(extrainfo)

    def setUp(self):
        def _cwd(filename):
            return os.path.sep.join([os.getcwd(), filename])

        self._networkstatusFile = _cwd('BridgeTests-networkstatus-bridges')
        self._serverDescriptorFile = _cwd('BridgeTests-bridge-descriptors')
        self._extrainfoFile = _cwd('BridgeTests-cached-extrainfo')

        self._writeDescriptorFiles(BRIDGE_NETWORKSTATUS,
                                   BRIDGE_SERVER_DESCRIPTOR,
                                   BRIDGE_EXTRAINFO)
        self._parseAllDescriptorFiles()

        self.bridge = bridges.Bridge()

    def tearDown(self):
        """Reset safelogging to its default (disabled) state, due to
        test_Bridge_str_with_safelogging changing it.
        """
        bridges.safelog.safe_logging = False

    def test_Bridge_nickname_del(self):
        """The del method for the nickname property should reset the nickname
        to None.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.assertEqual(self.bridge.nickname, "FourfoldQuirked")

        del(self.bridge.nickname)
        self.assertIsNone(self.bridge.nickname)
        self.assertIsNone(self.bridge._nickname)

    def test_Bridge_nickname_invalid(self):
        """The del method for the nickname property should reset the nickname
        to None.
        """
        # Create a networkstatus descriptor with an invalid nickname:
        filename = self._networkstatusFile + "-invalid"
        fh = open(filename, 'w')
        invalid = BRIDGE_NETWORKSTATUS.replace(
            "FourfoldQuirked",
            "ThisRouterNicknameContainsWayMoreThanNineteenBytes")
        fh.seek(0)
        fh.write(invalid)
        fh.flush()
        fh.close()

        self.assertRaises(InvalidRouterNickname,
                          descriptors.parseNetworkStatusFile,
                          filename)

    def test_Bridge_orport_del(self):
        """The del method for the orPort property should reset the orPort
        to None.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.assertEqual(self.bridge.orPort, 36489)

        del(self.bridge.orPort)
        self.assertIsNone(self.bridge.orPort)
        self.assertIsNone(self.bridge._orPort)

    def test_Bridge_str_without_safelogging(self):
        """The str() method of a Bridge should return an identifier for the
        Bridge, which should be different if safelogging is enabled.
        """
        bridges.safelog.safe_logging = False

        bridge = bridges.Bridge()
        bridge.updateFromNetworkStatus(self.networkstatus)

        identifier = str(bridge)
        self.assertEqual(identifier,
                         ''.join(['$', bridge.fingerprint,
                                  '~', bridge.nickname]))

    def test_Bridge_str_with_safelogging(self):
        """The str() method of a Bridge should return an identifier for the
        Bridge, which should be different if safelogging is enabled.
        """
        bridges.safelog.safe_logging = True

        bridge = bridges.Bridge()
        bridge.updateFromNetworkStatus(self.networkstatus)

        identifier = str(bridge)
        self.assertEqual(
            identifier,
            ''.join(['$$',
                     hashlib.sha1(bridge.fingerprint).hexdigest().upper(),
                     '~', bridge.nickname]))

    def test_Bridge_str_without_fingerprint(self):
        """The str() method of a Bridge should return an identifier for the
        Bridge, which should be different if the fingerprint is unknown.
        """
        bridge = bridges.Bridge()
        bridge.updateFromNetworkStatus(self.networkstatus)
        del(bridge.fingerprint)

        identifier = str(bridge)
        self.assertEqual(identifier,
                         ''.join(['$', '0'*40,
                                  '~', bridge.nickname]))

    def test_Bridge_updateFromNetworkStatus_IPv4_ORAddress(self):
        """Calling updateFromNetworkStatus() with a descriptor which has an
        IPv4 address as an additional ORAddress should result in a
        FutureWarning before continuing parsing.
        """
        # Add an additional IPv4 ORAddress:
        ns = BRIDGE_NETWORKSTATUS.replace(
            'a [6bf3:806b:78cd:d4b4:f6a7:4ced:cfad:dad4]:36488',
            'a [6bf3:806b:78cd:d4b4:f6a7:4ced:cfad:dad4]:36488\na 123.34.56.78:36488')
        self._writeNetworkstatus(ns)
        self._parseAllDescriptorFiles()

        self.assertWarns(
            FutureWarning,
            "Got IPv4 address in 'a'/'or-address' line! Descriptor format may have changed!",
            bridges.__file__,  # filename
            self.bridge.updateFromNetworkStatus,
            self.networkstatus)

        self.assertEqual(self.bridge.fingerprint,
                         '2C3225C4805331025E211F4B6E5BF45C333FDD2C')
        self.assertIn((ipaddr.IPAddress('123.34.56.78'), 36488, 4),
                      self.bridge.allVanillaAddresses)

    def test_Bridge_updateFromServerDescriptor(self):
        """ """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)

        self.assertEqual(self.bridge.fingerprint,
                         '2C3225C4805331025E211F4B6E5BF45C333FDD2C')

    def test_Bridge_updateFromServerDescriptor_no_networkstatus(self):
        """Parsing a server descriptor for a bridge which wasn't included in
        the networkstatus document from the BridgeAuthority should raise a
        ServerDescriptorWithoutNetworkstatus exception.
        """
        self.assertRaises(bridges.ServerDescriptorWithoutNetworkstatus,
                          self.bridge.updateFromServerDescriptor,
                          self.serverdescriptor)

    def test_Bridge_verifyExtraInfoSignature_good_signature(self):
        """Calling _verifyExtraInfoSignature() with a descriptor which has a
        good signature should return None.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.assertIsNone(self.bridge._verifyExtraInfoSignature(self.extrainfo))

    def test_Bridge_updateFromExtraInfoDescriptor(self):
        """Bridge.updateFromExtraInfoDescriptor() should add the expected
        number of pluggable transports.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.assertEqual(self.bridge.fingerprint,
                         '2C3225C4805331025E211F4B6E5BF45C333FDD2C')
        self.assertEqual(self.bridge.bandwidthObserved, None)
        self.assertEqual(len(self.bridge.transports), 0)

        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.assertEqual(self.bridge.fingerprint,
                         '2C3225C4805331025E211F4B6E5BF45C333FDD2C')
        self.assertEqual(self.bridge.bandwidthObserved, 1623207134)
        self.assertEqual(len(self.bridge.transports), 0)

        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)
        self.assertEqual(self.bridge.fingerprint,
                         '2C3225C4805331025E211F4B6E5BF45C333FDD2C')
        self.assertEqual(self.bridge.bandwidthObserved, 1623207134)
        self.assertEqual(len(self.bridge.transports), 4)

    def test_Bridge_updateFromExtraInfoDescriptor_bad_signature_changed(self):
        """Calling updateFromExtraInfoDescriptor() with a descriptor which
        has a bad signature should not continue to process the descriptor.
        """
        # Make the signature uppercased
        BEGIN_SIG = '-----BEGIN SIGNATURE-----'
        doc, sig = BRIDGE_EXTRAINFO.split(BEGIN_SIG)
        ei = BEGIN_SIG.join([doc, sig.upper()])
        self._writeExtrainfo(ei)
        self._parseAllDescriptorFiles()

        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        self.assertEqual(len(self.bridge.transports), 0)
        self.assertIsNone(self.bridge.descriptors['extrainfo'])

    def test_Bridge_updateFromExtraInfoDescriptor_pt_changed_port(self):
        """Calling updateFromExtraInfoDescriptor() with a descriptor which
        includes a different port for a known bridge with a known pluggable
        transport should update that transport.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        self.assertEqual(len(self.bridge.transports), 4)

        for pt in self.bridge.transports:
            if pt.methodname == 'obfs4':
                self.assertEqual(pt.address, ipaddr.IPv4Address('179.178.155.140'))
                self.assertEqual(pt.port, 36493)

        # Change the port of obfs4 transport in the extrainfo descriptor:
        transportline = self.extrainfo.transport['obfs4']
        self.extrainfo.transport['obfs4'] = (transportline[0],
                                             31337,
                                             transportline[2])
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        for pt in self.bridge.transports:
            if pt.methodname == 'obfs4':
                self.assertEqual(pt.address, ipaddr.IPv4Address('179.178.155.140'))
                self.assertEqual(pt.port, 31337)

    def test_Bridge_updateFromExtraInfoDescriptor_pt_changed_args(self):
        """Calling updateFromExtraInfoDescriptor() with a descriptor which
        includes different PT args for a known bridge with a known pluggable
        transport should update that transport.

        scramblesuit 179.178.155.140:36492 password=ABCDEFGHIJKLMNOPQRSTUVWXYZ234567
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        self.assertEqual(len(self.bridge.transports), 4)

        for pt in self.bridge.transports:
            if pt.methodname == 'scramblesuit':
                self.assertEqual(pt.address, ipaddr.IPv4Address('179.178.155.140'))
                self.assertEqual(pt.port, 36492)

        # Change the args of scramblesuit transport in the extrainfo descriptor:
        transportline = self.extrainfo.transport['scramblesuit']
        self.extrainfo.transport['scramblesuit'] = (transportline[0],
                                                    transportline[1],
                                                    ['password=PASSWORD'])
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        for pt in self.bridge.transports:
            if pt.methodname == 'scramblesuit':
                self.assertEqual(pt.address, ipaddr.IPv4Address('179.178.155.140'))
                self.assertEqual(pt.port, 36492)
                self.assertEqual(pt.arguments['password'], 'PASSWORD')

    def test_Bridge_updateFromExtraInfoDescriptor_pt_died(self):
        """Calling updateFromExtraInfoDescriptor() with a descriptor which
        doesn't include a previously-known transport should remove that
        transport.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        self.assertEqual(len(self.bridge.transports), 4)

        # Remove the obfs3 transport from the extrainfo descriptor:
        self.extrainfo.transport.pop('obfs3')
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        self.assertEqual(len(self.bridge.transports), 3)

        for pt in self.bridge.transports:
            self.failIfEqual(pt.methodname, 'obfs3')

    def test_Bridge_descriptorDigest(self):
        """Parsing a networkstatus descriptor should result in
        Bridge.descriptorDigest being set.
        """
        realdigest = "738115BB6ACEFE20FF0C96015FF2E5DFC0C64162"
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.assertEqual(self.bridge.descriptorDigest, realdigest)

    def test_Bridge_checkServerDescriptor(self):
        """Parsing a server descriptor when the bridge's networkstatus document
        didn't have a digest of the server descriptor should raise a
        MissingServerDescriptorDigest.
        """
        # Create a networkstatus descriptor without a server descriptor digest:
        filename = self._networkstatusFile + "-missing-digest"
        fh = open(filename, 'w')
        invalid = BRIDGE_NETWORKSTATUS.replace("c4EVu2rO/iD/DJYBX/Ll38DGQWI", "foo")
        fh.seek(0)
        fh.write(invalid)
        fh.flush()
        fh.close()

        realdigest = "738115BB6ACEFE20FF0C96015FF2E5DFC0C64162"

        #networkstatus = descriptors.parseNetworkStatusFile(filename)
        #self.bridge.updateFromNetworkStatus(networkstatus[0])
        #self.assertRaises(bridges.MissingServerDescriptorDigest,
        #                  self.bridge.updateFromNetworkStatus,
        #                  networkstatus[0])

    def test_Bridge_checkServerDescriptor_digest_mismatch_ns(self):
        """Parsing a server descriptor whose digest doesn't match the one given
        in the bridge's networkstatus document should raise a
        ServerDescriptorDigestMismatch.
        """
        # Create a networkstatus descriptor without a server descriptor digest:
        filename = self._networkstatusFile + "-mismatched-digest"
        fh = open(filename, 'w')
        invalid = BRIDGE_NETWORKSTATUS.replace("c4EVu2rO/iD/DJYBX/Ll38DGQWI",
                                               "c4EVu2r1/iD/DJYBX/Ll38DGQWI")
        fh.seek(0)
        fh.write(invalid)
        fh.flush()
        fh.close()

        realdigest = "738115BB6ACEFE20FF0C96015FF2E5DFC0C64162"
        networkstatus = descriptors.parseNetworkStatusFile(filename)
        self.bridge.updateFromNetworkStatus(networkstatus[0])
        #self.bridge.updateFromServerDescriptor(self.serverdescriptor)

        self.assertRaises(bridges.ServerDescriptorDigestMismatch,
                          self.bridge.updateFromServerDescriptor,
                          self.serverdescriptor)

    def test_Bridge_checkServerDescriptor_digest_mismatch_sd(self):
        """Parsing a server descriptor when the corresponding networkstatus
        descriptor didn't include a server bridge.descriptorDigest that matches
        should raise a ServerDescriptorDigestMismatch exception.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)

        self.bridge.descriptorDigest = 'deadbeef'
        self.assertRaises(bridges.ServerDescriptorDigestMismatch,
                          self.bridge._checkServerDescriptor,
                          self.serverdescriptor)

    def test_Bridge_checkServerDescriptor_digest_missing(self):
        """Parsing a server descriptor when the corresponding networkstatus
        descriptor didn't include a server bridge.descriptorDigest should raise
        a MissingServerDescriptorDigest exception.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)

        self.bridge.descriptorDigest = None
        self.assertRaises(bridges.MissingServerDescriptorDigest,
                          self.bridge._checkServerDescriptor,
                          self.serverdescriptor)

    def test_Bridge_assertOK(self):
        """If all orAddresses are okay, then assertOK() should return None."""
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)

        self.assertIsNone(self.bridge.assertOK())

    def test_Bridge_assertOK_all_bad_values(self):
        """If an orAddress has an IP address of 999.999.999.999 and a port of
        -1 and claims to be IPv5, then everything about it is bad and it should
        fail all the checks in assertOK(), then a MalformedBridgeInfo should be
        raised.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)

        # All values are bad (even though IPv5 is a thing):
        self.bridge.orAddresses.append(('999.999.999.999', -1, 5))
        self.assertRaises(bridges.MalformedBridgeInfo, self.bridge.assertOK)

    def test_Bridge_getBridgeLine_request_valid(self):
        """Calling getBridgeLine with a valid request should return a bridge
        line.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        line = self.bridge.getBridgeLine(request)

        self.assertIsNotNone(line)
        self.assertIn('179.178.155.140:36489', line)
        self.assertIn('2C3225C4805331025E211F4B6E5BF45C333FDD2C', line)

    def test_Bridge_getBridgeLine_request_invalid(self):
        """Calling getBridgeLine with an invalid request should return None."""
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(False)

        self.assertIsNone(self.bridge.getBridgeLine(request))

    def test_Bridge_getBridgeLine_no_vanilla_addresses(self):
        """Calling getBridgeLine() on a Bridge without any vanilla addresses
        should return None.
        """
        request = BridgeRequestBase()
        request.isValid(True)

        self.assertIsNone(self.bridge.getBridgeLine(request))

    def test_Bridge_getBridgeLine_request_without_block_in_IR(self):
        """Calling getBridgeLine() with a valid request for bridges not blocked
        in Iran should return a bridge line.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        request.withoutBlockInCountry('IR')
        line = self.bridge.getBridgeLine(request)

        self.assertIsNotNone(line)
        self.assertIn('179.178.155.140:36489', line)
        self.assertIn('2C3225C4805331025E211F4B6E5BF45C333FDD2C', line)

    def test_Bridge_getBridgeLine_IPv6(self):
        """Calling getBridgeLine() with a valid request for IPv6 bridges
        should return a bridge line.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        request.withIPv6()
        line = self.bridge.getBridgeLine(request)

        self.assertIsNotNone(line)
        self.assertTrue(
            line.startswith('[6bf3:806b:78cd:d4b4:f6a7:4ced:cfad:dad4]:36488'))
        self.assertNotIn('179.178.155.140:36493', line)
        self.assertIn('2C3225C4805331025E211F4B6E5BF45C333FDD2C', line)

    def test_Bridge_getBridgeLine_obfs4(self):
        """ """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        request.withPluggableTransportType('obfs4')
        line = self.bridge.getBridgeLine(request)

        self.assertIsNotNone(line)
        self.assertIn('179.178.155.140:36493', line)
        self.assertTrue(line.startswith('obfs4'))
        self.assertIn('iat-mode', line)
        self.assertIn('public-key', line)
        self.assertIn('node-id', line)

    def test_Bridge_getBridgeLine_obfs3_IPv6(self):
        """Calling getBridgeLine() with a request for IPv6 obfs3 bridges (when
        the Bridge doesn't have any) should raise a
        PluggableTransportUnavailable exception.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        request.withIPv6()
        request.withPluggableTransportType('obfs3')

        self.assertRaises(bridges.PluggableTransportUnavailable,
                          self.bridge.getBridgeLine,
                          request)

    def test_Bridge_getBridgeLine_googlygooglybegone(self):
        """Calling getBridgeLine() with a request for an unknown PT should
        raise a PluggableTransportUnavailable exception.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        request.withPluggableTransportType('googlygooglybegone')

        self.assertRaises(bridges.PluggableTransportUnavailable,
                          self.bridge.getBridgeLine,
                          request)

    def test_Bridge_getBridgeLine_bridge_prefix(self):
        """Calling getBridgeLine() with bridgePrefix=True should prefix the
        returned bridge line with 'Bridge '.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        line = self.bridge.getBridgeLine(request, bridgePrefix=True)

        self.assertIsNotNone(line)
        self.assertIn('179.178.155.140:36489', line)
        self.assertIn('2C3225C4805331025E211F4B6E5BF45C333FDD2C', line)
        self.assertTrue(line.startswith('Bridge'))

    def test_Bridge_getBridgeLine_no_include_fingerprint(self):
        """Calling getBridgeLine() with includeFingerprint=False should return
        a bridge line without a fingerprint.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        request = BridgeRequestBase()
        request.isValid(True)
        line = self.bridge.getBridgeLine(request, includeFingerprint=False)

        self.assertIsNotNone(line)
        self.assertIn('179.178.155.140:36489', line)
        self.assertNotIn('2C3225C4805331025E211F4B6E5BF45C333FDD2C', line)

    def test_Bridge_getNetworkstatusLastPublished(self):
        """Calling getNetworkstatusLastPublished() should tell us the last
        published time of the Bridge's server-descriptor.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)

        published = self.bridge.getNetworkstatusLastPublished()
        self.assertIsNotNone(published)
        self.assertIsInstance(published, datetime.datetime)
        self.assertEqual(str(published), '2014-12-22 21:51:27')

    def test_Bridge_getDescriptorLastPublished(self):
        """Calling getDescriptorLastPublished() should tell us the last
        published time of the Bridge's server-descriptor.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)

        published = self.bridge.getDescriptorLastPublished()
        self.assertIsNotNone(published)
        self.assertIsInstance(published, datetime.datetime)
        self.assertEqual(str(published), '2014-12-22 21:51:27')

    def test_Bridge_getExtrainfoLastPublished(self):
        """Calling getNetworkstatusLastPublished() should tell us the last
        published time of the Bridge's server-descriptor.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        published = self.bridge.getExtrainfoLastPublished()
        self.assertIsNotNone(published)
        self.assertIsInstance(published, datetime.datetime)
        self.assertEqual(str(published), '2014-12-22 21:51:27')

    def test_Bridge_isBlockedIn_IS(self):
        """Calling isBlockedIn('IS') should return False when the bridge isn't
        blocked in Iceland.
        """
        self.assertFalse(self.bridge.isBlockedIn('IS'))

    def test_Bridge_setBlockedIn_CN_obfs2(self):
        """Calling setBlockedIn('CN', 'obfs2') should mark all obfs2 transports
        of the bridge as being blocked in CN.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        self.bridge.setBlockedIn('CN', methodname='obfs2')
        self.assertTrue(self.bridge.isBlockedIn('CN'))

    def test_Bridge_setBlockedIn_IR_address(self):
        """Calling setBlockedIn('IR', address) should mark all matching
        addresses of the bridge as being blocked in IR.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        self.bridge.setBlockedIn('IR', address='179.178.155.140')
        self.assertTrue(self.bridge.isBlockedIn('ir'))
        self.assertFalse(self.bridge.isBlockedIn('cn'))

    def test_Bridge_setBlockedIn_GB_address_port(self):
        """Calling setBlockedIn('GB', address, port) should mark all matching
        addresses:port pairs of the bridge as being blocked in GB.
        """
        self.bridge.updateFromNetworkStatus(self.networkstatus)
        self.bridge.updateFromServerDescriptor(self.serverdescriptor)
        self.bridge.updateFromExtraInfoDescriptor(self.extrainfo)

        # Should block the obfs4 bridge:
        self.bridge.setBlockedIn('GB', address='179.178.155.140', port=36493)
        self.assertTrue(self.bridge.isBlockedIn('GB'))
        self.assertTrue(self.bridge.isBlockedIn('gb'))
        self.assertTrue(self.bridge.transportIsBlockedIn('GB', 'obfs4'))
        self.assertTrue(self.bridge.addressIsBlockedIn('GB', '179.178.155.140', 36493))
        self.assertFalse(self.bridge.addressIsBlockedIn('gb', '179.178.155.140', 36488))



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""deprecated ― functions and classes which have been removed from the
production code but are kept in order to be used in regression testing.
"""

import ipaddr
import re

from twisted.python import deprecate
from twisted.python.versions import Version


PORTSPEC_LENGTH = 16

re_ipv6 = re.compile("\[([a-fA-F0-9:]+)\]:(.*$)")
re_ipv4 = re.compile("((?:\d{1,3}\.?){4}):(.*$)")

deprecate.deprecatedModuleAttribute(
    Version('bridgedb', 0, 0, 1),
    "Removed due to 'bridgedb.Bridges.PortList' being moved to "\
    "'bridgedb.parse.addr.PortList.",
    "bridgedb.Bridges",
    "PORTSPEC_LENGTH")

deprecate.deprecatedModuleAttribute(
    Version('bridgedb', 0, 0, 1),
    "Attribute 'bridgedb.Bridges.re_ipv4' was removed due to "\
    "'bridgedb.Bridges.parseORAddressLine' moving to "\
    "'bridgedb.parse.networkstatus.",
    "bridgedb.Bridges",
    "re_ipv4")

deprecate.deprecatedModuleAttribute(
    Version('bridgedb', 0, 0, 1),
    "Attribute 'bridgedb.Bridges.re_ipv6' was removed due to "\
    "'bridgedb.Bridges.parseORAddressLine' moving to "\
    "'bridgedb.parse.networkstatus.",
    "bridgedb.Bridges",
    "re_ipv6")

deprecate.deprecatedModuleAttribute(
    Version('bridgedb', 0, 2, 3),
    ("Removed due to 'bridgedb.Bridges.HEX_FP_LEN' being moved to "
     "'bridgedb.parse.fingerprint.HEX_FINGERPRINT_LEN."),
    "bridgedb.Bridges", "HEX_FP_LEN")

deprecate.deprecatedModuleAttribute(
    Version('bridgedb', 0, 2, 3),
    ("Removed due to 'bridgedb.Bridges.toHex' being moved to "
     "'bridgedb.parse.fingerprint.toHex."),
    "bridgedb.Bridges", "toHex")

deprecate.deprecatedModuleAttribute(
    Version('bridgedb', 0, 2, 3),
    ("Removed due to 'bridgedb.Bridges.fromHex' being moved to "
     "'bridgedb.parse.fingerprint.fromHex."),
    "bridgedb.Bridges", "fromHex")


@deprecate.deprecated(
    Version('bridgedb', 0, 0, 1),
    replacement='bridgedb.parse.networkstatus.parseALine')
def parseORAddressLine(line):
    """Deprecated :func:`bridgedb.Bridges.parseORAddressLine`, removed in
    bridgedb-0.1.0, in commit 1f111e5.

    This function and the newer parsers from :mod:`bridgedb.parse.netstatus`
    are alternately :api:`~twisted.python.monkey.MonkeyPatcher.patch`ed into
    the :mod:`old unittests <bridgedb.Tests>`, so that the later functions as
    a suite of regression tests.
    """
    address = None
    portlist = None
    # try regexp to discover ip version
    for regex in [re_ipv4, re_ipv6]:
        m = regex.match(line)
        if m:
            # get an address and portspec, or raise ParseError
            try:
                address  = ipaddr.IPAddress(m.group(1))
                portlist = PortList(m.group(2))
            except (IndexError, ValueError): raise ParseORAddressError(line)

    # return a valid address, portlist or raise ParseORAddressError
    if address and portlist and len(portlist): return address,portlist
    raise ParseORAddressError(line)

@deprecate.deprecated(Version('bridgedb', 0, 2, 3),
                      replacement='bridgedb.parse.fingerprint.isValidFingerprint')
def is_valid_fingerprint(fp):
    """Return true iff fp in the right format to be a hex fingerprint
       of a Tor server.
    """
    if len(fp) != HEX_FP_LEN:
        return False
    try:
        fromHex(fp)
    except TypeError:
        return False
    else:
        return True

@deprecate.deprecated(Version('bridgedb', 0, 2, 4),
                      replacement='bridgedb.parse.descriptors.parseExtraInfoFiles')
def parseExtraInfoFile(f):
    """
    parses lines in Bridges extra-info documents.
    returns an object whose type corresponds to the
    relevant set of extra-info lines.

    presently supported lines and the accompanying type are:

        { 'transport': PluggableTransport, }

    'transport' lines (torspec.git/proposals/180-pluggable-transport.txt)

        Bridges put the 'transport' lines in their extra-info documents.
        the format is:

            transport SP <methodname> SP <address:port> [SP arglist] NL
    """

    ID = None
    for line in f:
        line = line.strip()

        argdict = {}

        # do we need to skip 'opt' here?
        # if line.startswith("opt "):
        #     line = line[4:]

        # get the bridge ID ?
        if line.startswith("extra-info "): #XXX: get the router ID
            line = line[11:]
            (nickname, ID) = line.split()
            logging.debug("  Parsed Nickname: %s", nickname)
            if isValidFingerprint(ID):
                logging.debug("  Parsed fingerprint: %s", ID)
                ID = fromHex(ID)
            else:
                logging.debug("  Parsed invalid fingerprint: %s", ID)

        # get the transport line
        if ID and line.startswith("transport "):
            fields = line[10:].split()

            if len(fields) >= 3:
                argdict = {}
                # PT argumentss are comma-separated in the extrainfo
                # descriptors. While there *shouldn't* be anything after them
                # that was separated by a space (and hence would wind up being
                # in a different `field`), if there was we'll join it to the
                # rest of the PT arguments with a comma so that they are
                # parsed as if they were PT arguments as well:
                allargs = ','.join(fields[2:])
                for arg in allargs.split(','):
                    try:
                        k, v = arg.split('=')
                    except ValueError:
                        logging.warn("  Couldn't parse K=V from PT arg: %r" % arg)
                    else:
                        logging.debug("  Parsed PT Argument: %s: %s" % (k, v))
                        argdict[k] = v

            # get the required fields, method name and address
            if len(fields) >= 2:
                # get the method name
                # Method names must be C identifiers
                for regex in [re_ipv4, re_ipv6]:
                    try:
                        method_name = re.match('[_a-zA-Z][_a-zA-Z0-9]*',fields[0]).group()
                        m = regex.match(fields[1])
                        address  = ipaddr.IPAddress(m.group(1))
                        port = int(m.group(2))
                        logging.debug("  Parsed Transport: %s %s:%d"
                                      % (method_name, address, port))
                        yield ID, method_name, address, port, argdict
                    except (IndexError, ValueError, AttributeError):
                        # skip this line
                        continue

        # end of descriptor is defined how?
        if ID and line.startswith("router-signature"):
            ID = None

@deprecate.deprecated(Version('bridgedb', 0, 2, 4),
                      replacement='bridgedb.parse.descriptors.parseNetworkStatusFile')
def parseStatusFile(networkstatusFile):
    """Parse entries in a bridge networkstatus file.

    :type networkstatusFile: A file-like object.
    :param networkstatusFile: A file containing `@type bridge-networkstatus` documents.
    """
    (nickname, ID, descDigest, timestamp,
     ORaddr, ORport, dirport, addr, portlist) = (None for x in xrange(9))
    running = stable = False
    parsedORAddressLines = 0
    or_addresses = {}

    for line in networkstatusFile:
        line = line.strip()
        if line.startswith("opt "):
            line = line[4:]

        if line.startswith("r "):
            (nickname, ID, descDigest, timestamp,
             ORaddr, ORport, dirport) = networkstatus.parseRLine(line)
            hexID = toHex(ID)
            logging.debug("Parsed networkstatus line:")
            logging.debug("  Nickname:   %s" % nickname)
            logging.debug("  Identity:   %s" % hexID)
            if descDigest:
                descDigest = toHex(descDigest)
                logging.debug("  Descriptor: {0}".format(descDigest))
                logging.debug("  Timestamp:  {0}".format(timestamp))
                logging.debug("  ORAddress:  {0}".format(ORaddr))
                logging.debug("  ORport:     {0}".format(ORport))
                logging.debug("  dirport:    {0}".format(dirport))

        elif ID and line.startswith("a "):
            try:
                addr, portlist = networkstatus.parseALine(line, toHex(ID))
            except networkstatus.NetworkstatusParsingError as error:
                logging.error(error)
            else:
                if (addr is not None) and (portlist is not None):
                    try:
                        or_addresses[addr].add(portlist)
                    except (KeyError, AttributeError):
                        or_addresses[addr] = portlist
                    parsedORAddressLines += 1

        elif ID and timestamp and line.startswith("s "):
            running, stable = networkstatus.parseSLine(line)
            logging.debug("Bridges.parseStatusFile(): "
                          "yielding %s nickname=%s descDigest=%s "
                          "running=%s stable=%s oraddr=%s orport=%s "
                          "oraddrs=%s ts=%s"
                          % (hexID, nickname, descDigest, running,
                             stable, ORaddr, ORport, or_addresses,
                             timestamp))
            yield (ID, nickname, descDigest, running, stable,
                   ipaddr.IPAddress(ORaddr), ORport,
                   or_addresses, timestamp)

            (nickname, ID, descDigest, timestamp, ORaddr, ORport, dirport,
             addr, portlist, hexID) = (None for x in xrange(10))
            running = stable = False
            or_addresses = {}

    logging.debug("Total ORAddress lines parsed from '%s': %d"
                  % (networkstatusFile.name, parsedORAddressLines))


@deprecate.deprecated(
    Version('bridgedb', 0, 0, 1),
    replacement='bridgedb.parse.networkstatus.NetworkstatusParsingError')
class ParseORAddressError(Exception):
    def __init__(self, line=None):
        msg = "Invalid or-address line"
        if line:
            msg += ": {0}".format(line)
        Exception.__init__(self, msg)


@deprecate.deprecated(
    Version('bridgedb', 0, 2, 4),
    replacement='bridgedb.bridges.Bridge')
class Bridge(object):
    """Holds information for a single bridge, along with any Pluggable
    Transports it is also running.

    :attr str nickname: The bridge's nickname.  Not currently used.
    :attr ip: (:class:`ipaddr.IPAddress`) The bridge's IPv4 address, specified
        on the 'r'-line in a networkstatus document.
    :attr int orport: The bridge's OR port.
    :attr dict or_addresses: The bridges alternate IP addresses. The keys
        should be instances of ``ipaddr.IPAddress``, and the value should be a
        :class:`bridgedb.parse.addr.PortList` for the port(s) on which that
        address is listening.
    :attr list transports: List of :class:`PluggableTransport` instances for
        each PT which the bridge supports.
    :attr str fingerprint: The bridge's identity digest, in lowercase hex,
        without whitespace.
    :attr bool running: ``True``, if this bridge was given the ``Running`` flag.
    :attr bool stable: ``True``, if this bridge was given the ``Stable`` flag.
    :attr dict blockingCountries: A dictionary whose keys are strings of
        ``"IP:port"`` pairs, and the keys are lists of two letter country
        codes which block that IP:port. For example::
            {"1.2.3.4:9001": ['sk', 'us', 'ir', 'cn']}
    :attr str desc_digest: SHA1 hexdigest of the bridge's descriptor as
        defined in the networkstatus document.
    :attr str ei_digest: SHA1 hexdigest of the bridge's extra-info document as
        given in the bridge's descriptor, corresponding to desc_digest.
    :attr bool verified: Did we receive the descriptor for this bridge that
        was specified in the networkstatus?
    """
    def __init__(self, nickname, ip, orport, fingerprint=None, id_digest=None,
                 or_addresses=None, transports=None):
        """Create a new Bridge. One of fingerprint and id_digest must be set.
        """
        self.nickname = nickname
        self.ip = ip
        self.orport = orport
        if not or_addresses: or_addresses = {}
        self.or_addresses = or_addresses
        if not transports: transports = []
        self.transports = transports
        self.running = self.stable = None
        self.blockingCountries = {}
        self.desc_digest = None
        self.ei_digest = None
        self.verified = False

        if id_digest is not None:
            assert fingerprint is None
            if len(id_digest) != DIGEST_LEN:
                raise TypeError("Bridge with invalid ID")
            self.fingerprint = toHex(id_digest)
        elif fingerprint is not None:
            if not isValidFingerprint(fingerprint):
                raise TypeError("Bridge with invalid fingerprint (%r)"%
                                fingerprint)
            self.fingerprint = fingerprint.lower()
        else:
            raise TypeError("Bridge with no ID")

    def setDescriptorDigest(self, digest):
        """Set the descriptor digest, specified in the NS."""
        self.desc_digest = digest

    def setExtraInfoDigest(self, digest):
        """Set the extra-info digest, specified in the descriptor."""
        self.ei_digest = digest

    def setVerified(self):
        """Call when the bridge's descriptor is parsed"""
        self.verified = True

    def isVerified(self):
        """Returns the truthiness of ``verified``"""
        return self.verified

    def getID(self):
        """Return the bridge's identity digest."""
        return fromHex(self.fingerprint)

    def __repr__(self):
        """Return a piece of python that evaluates to this bridge."""
        if self.or_addresses:
            return "Bridge(%r,%r,%d,%r,or_addresses=%s)"%(
                self.nickname, self.ip, self.orport, self.fingerprint,
                self.or_addresses)
        return "Bridge(%r,%r,%d,%r)"%(
            self.nickname, self.ip, self.orport, self.fingerprint)

    def getConfigLine(self, includeFingerprint=False, addressClass=None,
            request=None, transport=None):
        """Returns a valid bridge line for inclusion in a torrc.

        :param bool includeFingerprint: If ``True``, include the
            ``fingerprint`` of this :class:`Bridge` in the returned bridge
            line.
        :param DOCDOC addressClass: Type of address to choose.
        :param str request: A string unique to this request e.g. email-address
            or ``uniformMap(ip)`` or ``'default'``.
        :param str transport: A pluggable transport method name.
        """

        if not request: request = 'default'
        digest = getHMACFunc('Order-Or-Addresses')(request)
        pos = long(digest[:8], 16) # lower 8 bytes -> long

        # default address type
        if not addressClass: addressClass = ipaddr.IPv4Address

        # pluggable transports
        if transport:
            # filter by 'methodname'
            transports = filter(lambda x: transport == x.methodname,
                    self.transports)
            # filter by 'addressClass'
            transports = filter(lambda x: isinstance(x.address, addressClass),
                    transports)
            if transports:
                pt = transports[pos % len(transports)]
                return pt.getTransportLine(includeFingerprint)

        # filter addresses by address class
        addresses = filter(lambda x: isinstance(x[0], addressClass),
                self.or_addresses.items())

        # default ip, orport should get a chance at being selected
        if isinstance(self.ip, addressClass):
            addresses.insert(0,(self.ip, addr.PortList(self.orport)))

        if addresses:
            address,portlist = addresses[pos % len(addresses)]
            if isinstance(address, ipaddr.IPv6Address): ip = "[%s]"%address
            else: ip = "%s"%address
            orport = portlist[pos % len(portlist)]

            if includeFingerprint:
                return "%s:%d %s" % (ip, orport, self.fingerprint)
            else:
                return "%s:%d" % (ip, orport)

    def getAllConfigLines(self,includeFingerprint=False):
        """Generator. Iterate over all valid config lines for this bridge."""
        for address,portlist in self.or_addresses.items():
            if type(address) is ipaddr.IPv6Address:
                ip = "[%s]" % address
            else:
                ip = "%s" % address

            for orport in portlist:
                if includeFingerprint:
                    yield "bridge %s:%d %s" % (ip,orport,self.fingerprint)
                else:
                    yield "bridge %s:%d" % (ip,orport)
        for pt in self.transports:
            yield pt.getTransportLine(includeFingerprints)


    def assertOK(self):
        assert is_valid_ip(self.ip)
        assert isValidFingerprint(self.fingerprint)
        assert 1 <= self.orport <= 65535
        if self.or_addresses:
            for address, portlist in self.or_addresses.items():
                assert is_valid_ip(address)
                for port in portlist:
                    assert type(port) is int
                    assert 1 <= port <= 65535

    def setStatus(self, running=None, stable=None):
        if running is not None:
            self.running = running
        if stable is not None:
            self.stable = stable

    def isBlocked(self, countryCode, addressClass, methodname=None):
        """ if at least one address:port of the selected addressClass and
        (optional) transport type is not blocked in countryCode, return True
        """
        # 1) transport is specified
        if methodname is not None:
            for transport in self.transports:
                key = "%s:%s" % (transport.address, transport.port)
                if (isinstance(transport.address, addressClass)
                        and transport.methodname.lower() == methodname.lower()):
                    try:
                        if countryCode not in self.blockingCountries[key]:
                            return False
                    except KeyError:
                        return False # no blocklist
            return True
        # 2) no transport specified (default)
        else:
            # 3) check primary ip, port
            # XXX: could be more elegant if ip,orport were not special case
            if isinstance(self.ip, addressClass):
                key = "%s:%s" % (self.ip, self.orport)
                try:
                    if countryCode not in self.blockingCountries[key]:
                        return False
                except KeyError: return False # no blocklist

            # 4) check or addresses
            for address,portlist in self.or_addresses.items():
                if isinstance(address, addressClass):
                    # check each port
                    for port in portlist:
                        key = "%s:%s" % (address, port)
                        try:
                            if countryCode not in self.blockingCountries[key]:
                                return False
                        except KeyError: return False # no blocklist
            return True

    # Bridge Stability (#5482) properties.
    @property
    def familiar(self):
        """
        A bridge is 'familiar' if 1/8 of all active bridges have appeared
        more recently than it, or if it has been around for a Weighted Time of 8 days.
        """
        with bridgedb.Storage.getDB() as db:
            return db.getBridgeHistory(self.fingerprint).familiar

    @property
    def wfu(self):
        """Weighted Fractional Uptime"""
        with bridgedb.Storage.getDB() as db:
            return db.getBridgeHistory(self.fingerprint).weightedFractionalUptime

    @property
    def weightedTime(self):
        """Weighted Time"""
        with bridgedb.Storage.getDB() as db:
            return db.getBridgeHistory(self.fingerprint).weightedTime

    @property
    def wmtbac(self):
        """Weighted Mean Time Between Address Change"""
        with bridgedb.Storage.getDB() as db:
            return db.getBridgeHistory(self.fingerprint).wmtbac

    @property
    def tosa(self):
        """the Time On Same Address (TOSA)"""
        with bridgedb.Storage.getDB() as db:
            return db.getBridgeHistory(self.fingerprint).tosa

    @property
    def weightedUptime(self):
        """Weighted Uptime"""
        with bridgedb.Storage.getDB() as db:
            return db.getBridgeHistory(self.fingerprint).weightedUptime


@deprecate.deprecated(
    Version('bridgedb', 0, 2, 4),
    replacement='bridgedb.bridges.PluggableTransport')
class PluggableTransport(object):
    """A PT with reference to the parent bridge on which it is running.

    Deprecated :class:`bridgedb.Bridges.PluggableTransport`, replaced in
    bridgedb-0.2.4, by :class:`bridgedb.bridges.PluggableTransport`.
    """

    def __init__(self, bridge, methodname, address, port, argdict=None):
        """Create a ``PluggableTransport`` describing a PT running on a bridge.

        Pluggable transports are described within a bridge's ``@type
        bridge-extrainfo`` descriptor, see the ``Specifications: Client
        behavior`` section and the ``TOR_PT_SERVER_TRANSPORT_OPTIONS``
        description in pt-spec.txt_ for additional specification.

        :type bridge: :class:`Bridge`
        :param bridge: The parent bridge running this pluggable transport
            instance, i.e. the main ORPort bridge whose
            ``@type bridge-server-descriptor`` contains a hash digest for a
            ``@type bridge-extrainfo-document``, the latter of which contains
            the parameter of this pluggable transport in its ``transport``
            line.

        :param str methodname: The canonical "name" for this pluggable
            transport, i.e. the one which would be specified in a torrc
            file. For example, ``"obfs2"``, ``"obfs3"``, ``"scramblesuit"``
            would all be pluggable transport method names.

        :param str address: The IP address of the transport. Currently (as of
            20 March 2014), there are no known, widely-deployed pluggable
            transports which support IPv6. Ergo, this is very likely going to
            be an IPv4 address.

        :param int port: A integer specifying the port which this pluggable
            transport is listening on. (This should likely be whatever port the
            bridge specified in its ``ServerTransportPlugin`` torrc line,
            unless the pluggable transport is running in "managed" mode.)

        :param dict argdict: Some PTs can take additional arguments, which
            must be distributed to the client out-of-band. These are present
            in the ``@type bridge-extrainfo-document``, in the ``transport``
            line like so::

                METHOD SP ADDR ":" PORT SP [K=V[,K=V[,K=V[…]]]]

            where K is the **argdict** key, and V is the value. For example,
            in the case of ``scramblesuit``, for which the client must supply
            a shared secret to the ``scramblesuit`` instance running on the
            bridge, the **argdict** would be something like::

                {'password': 'NEQGQYLUMUQGK5TFOJ4XI2DJNZTS4LRO'}

        .. _pt-spec.txt:
             https://gitweb.torproject.org/torspec.git/blob/HEAD:/pt-spec.txt
        """
        #XXX: assert are disabled with python -O
        assert isinstance(bridge, Bridge)
        assert type(address) in (ipaddr.IPv4Address, ipaddr.IPv6Address)
        assert type(port) is int
        assert (0 < port < 65536)
        assert type(methodname) is str

        self.bridge = bridge
        self.address = address
        self.port = port
        self.methodname = methodname
        if type(argdict) is dict:
            self.argdict = argdict
        else: self.argdict = {}

    def getTransportLine(self, includeFingerprint=False, bridgePrefix=False):
        """Get a torrc line for this pluggable transport.

        This method does not return lines which are prefixed with the word
        'bridge', as they would be in a torrc file. Instead, lines returned
        look like this:

        obfs3 245.102.100.252:23619 59ca743e89b508e16b8c7c6d2290efdfd14eea98

        :param bool includeFingerprints: If ``True``, include the digest of
            this bridges public identity key in the torrc line.
        :param bool bridgePrefix: If ``True``, add ``'Bridge '`` to the
             beginning of each returned line (suitable for pasting directly
             into a torrc file).
        :rtype: str
        :returns: A configuration line for adding this pluggable transport
            into a torrc file.
        """
        sections = []

        if bridgePrefix:
            sections.append('Bridge')

        if isinstance(self.address, ipaddr.IPv6Address):
            host = "%s [%s]:%d" % (self.methodname, self.address, self.port)
        else:
            host = "%s %s:%d" % (self.methodname, self.address, self.port)
        sections.append(host)

        if includeFingerprint:
            sections.append(self.bridge.fingerprint)

        args = " ".join(["%s=%s" % (k, v) for k, v in self.argdict.items()])
        sections.append(args)

        line = ' '.join(sections)
        return line


@deprecate.deprecated(
    Version('bridgedb', 0, 0, 1),
    replacement='bridgedb.parse.addr.PortList')
class PortList:
    """Deprecated :class:`bridgedb.Bridges.PortList`, replaced in
    bridgedb-0.1.0, in commit 1f111e5, by
    :class:`bridgedb.parse.addr.PortList`.

    This class and the newer class from :mod:`bridgedb.parse.addr` are
    alternately :api:`~twisted.python.monkey.MonkeyPatcher.patch`ed into the
    :mod:`old unittests <bridgedb.Tests>`, so that the later functions as a
    suite of regression tests.
    """
    def __init__(self, *args, **kwargs):
        self.ports = set()
        self.add(*args)

    def _sanitycheck(self, val):
        #XXX: if debug=False this is disabled. bad!
        assert type(val) is int
        assert(0 < val <= 65535)

    def __contains__(self, val1):
        return val1 in self.ports

    def add(self, *args):
        PORTSPEC_LEN = 16
        for arg in args:
            try:
                if type(arg) is str:
                    ports = set([int(p) for p in arg.split(',')][:PORTSPEC_LEN])
                    [self._sanitycheck(p) for p in ports]
                    self.ports.update(ports)
                if type(arg) is int:
                    self._sanitycheck(arg)
                    self.ports.update([arg])
                if type(arg) is PortList:
                    self.add(list(arg.ports))
            except AssertionError: raise ValueError
            except ValueError: raise

    def __iter__(self):
        return self.ports.__iter__()

    def __str__(self):
        s = ""
        for p in self.ports:
            s += "".join(",%s"%p)
        return s.lstrip(",")

    def __repr__(self):
        return "PortList('%s')" % self.__str__()

    def __len__(self):
        return len(self.ports)

    def __getitem__(self, x):
        return list(self.ports)[x]



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013 Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-clause BSD, see included LICENSE for information

'''Package containing modules for parsing data.

.. py:module:: bridgedb.parse
    :synopsis: Package containing modules for parsing data.
'''

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import binascii


class InvalidBase64(ValueError):
    """Raised if parsing or decoding cannot continue due to invalid base64."""


def padBase64(b64string):
    """Re-add any stripped equals sign character padding to a b64 string.

    :param string b64string: A base64-encoded string which might have had its
        trailing equals sign (``=``) padding removed.
    :raises ValueError: if there was any error while manipulating the string.
    :returns: A properly-padded (according to the base64 spec: :rfc:`4648`)
        string.
    """
    addchars  = 0
    try:
        b64string = b64string.strip()
        remainder = len(b64string) % 4
        if 2 <= remainder <= 3:
            addchars = 4 - remainder
    except AttributeError as error:
        raise ValueError(error)
    else:
        if not addchars:
            raise ValueError("Invalid base64-encoded string: %r" % b64string)
        b64string += '=' * addchars

    return b64string

def parseUnpaddedBase64(field):
    """Parse an unpadded, base64-encoded field.

    The **field** will be re-padded, if need be, and then base64 decoded.

    :param str field: Should be some base64-encoded thing, with any trailing
        ``=``-characters removed.
    :raises InvalidBase64: if there is an error in either unpadding or decoding
        **field**.
    :rtype: str
    :returns: The base64-decoded **field**.
    """
    if field.endswith('='):
        raise InvalidBase64("Unpadded, base64-encoded networkstatus field "\
                            "must not end with '=': %r" % field)

    try:
        paddedField = padBase64(field)  # Add the trailing equals sign back in
    except ValueError as error:
        raise InvalidBase64(error)

    debasedField = binascii.a2b_base64(paddedField)
    if not debasedField:
        raise InvalidBase64("Base64-encoded networkstatus field %r is invalid!"
                            % field)

    return debasedField
